/***************************************************************************
                          board.cpp  -  description
                             -------------------
    begin                : Sat Sep 29 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "board.h"
//#include <kiconeffect.h>
#include <qpainter.h>
#include <stdlib.h>
#include <assert.h>
#include <qbitmap.h>
#include <qpe/qpeapplication.h>
bool inMiddle;
board::board(QWidget *parent, const char *name, resource *Rsrc, logic *Lgc ) : QWidget(parent,name)
{
	inMiddle = false;
	Resource = Rsrc;
	Logic = Lgc;
	paused = FALSE;
	localArmy = WHITE;
	flip = FALSE;
	dragging = Null;
	premoveFrom = Null;
	premoveTo = Null;
	cache = Resource->pixCache;
	myself.resize( Resource->ThemeSize * 8, Resource->ThemeSize * 8);
	myself.fill();
       // QPEApplication::setStylusOperation(this,QPEApplication::RightOnHold);
	show();
}
board::~board()
{
}
///////////////////////////////////////
//
//	board::ResizeBoard				( SLOT )
//
///////////////////////////////////////
void board::ResizeBoard( void )
{
	qDebug("board::ResizeBoard");
	myself.resize( Resource->ThemeSize * 8, Resource->ThemeSize * 8);
	myself.fill();
	setFixedSize( Resource->ThemeSize * 8, Resource->ThemeSize * 8);
	redrawAll();
}
///////////////////////////////////////
//
//	board::coords
//
///////////////////////////////////////
QPoint board::coords( const int rank, const int file )
{
	QPoint tmp;
	bool orientation( Resource->OPTION_Board_Orientation );
	//if( localArmy != WHITE ) orientation = !orientation;
	if( flip ) orientation = !orientation;
	if( orientation == 0 )
	{
		tmp.setX( Resource->ThemeSize * file );
		tmp.setY( Resource->ThemeSize * ( 7 - rank ) );
	}
	else
	{
		tmp.setX( Resource->ThemeSize * ( 7 - file ) );
		tmp.setY( Resource->ThemeSize * rank );
	}
	return tmp;
}
///////////////////////////////////////
//
//	board::position
//
///////////////////////////////////////
int board::position( const QPoint point )
{
	int file, rank;
	bool orientation( Resource->OPTION_Board_Orientation );

//	if( localArmy != WHITE ) orientation = !orientation;
	if( flip ) orientation = !orientation;
	if( orientation == 0 )
	{
		file = point.x() / Resource->ThemeSize;
		rank = 7 - ( point.y() / Resource->ThemeSize );
	}
	else
	{
		file = 7 - ( point.x() / Resource->ThemeSize );
		rank = point.y() / Resource->ThemeSize;
	}
	return ( ( rank * 8 ) + file );
}
///////////////////////////////////////
//
//	board::redrawAll
//
///////////////////////////////////////
void board::redrawAll( void )
{
	char tmp(0);
	while( tmp < 64 ) drawPosition( tmp++ );
	commit();
}
///////////////////////////////////////
//
//	board::commit
//
///////////////////////////////////////
void board::commit( void )
{
	if( dragging != Null ) drawDrag();
	setBackgroundPixmap( myself );
}
///////////////////////////////////////
//
//	board::drawPosition
//
///////////////////////////////////////
void board::drawPosition( const int pos )
{
	drawPosition( ( pos / 8 ), ( pos % 8 ) );
}
void board::drawPosition( const int rank, const int file )
{
int pos( ( rank * 8 ) + file );
	QPixmap buffer;
	QString cacheName;
	QImage tempImage;

	if( ( pos < 0 ) || ( pos > 63 ) )
		return;
	/*
			Build the cache ref name
	*/
	cacheName = "$knights_";
	if( color( rank, file ) ) cacheName += "L";
	else cacheName += "D";
	if( !paused )
	{
		switch( Logic->current[pos].Note )
		{
			case NOTE_SELECT:
			case NOTE_HIGHLIGHT:
				cacheName += "S";
				break;
  		case NOTE_MOVE:
  		case NOTE_CASTLE:
  		case NOTE_PAWN_DOUBLE:
  			cacheName += "M";
  			break;
  		case NOTE_ATTACK:
  		case NOTE_ENPASSANT:
  			cacheName += "A";
  			break;
  		default:
  			cacheName += " ";
  	}
  	if( ( Logic->current[pos].ManPtr != Null ) && ( dragging != pos ) )
  	{
  		if( Logic->chessman[ Logic->current[pos].ManPtr ].Army == WHITE ) cacheName += "W";
  		else cacheName += "B";
    	switch( Logic->chessman[Logic->current[pos].ManPtr].Type )
    	{
    		case King:
    					cacheName += "K";
    			break;
    		case Queen:
    					cacheName += "Q";
    			break;
    		case Bishop:
    					cacheName += "B";
    			break;
    		case Knight:
    					cacheName += "N";
    			break;
    		case Rook:
    					cacheName += "R";
    			break;
    		case Pawn:
    					cacheName += "P";
    			break;
    		default:
    			break;
    	}
  	}
	}
	else
 		cacheName += " ";
	if( cache->find( cacheName, buffer ) )
	{
		/*
				Cache Hit... no need to redraw
		*/
		if( Resource->OPTION_Show_Coord ) {
			         drawCoords( &buffer, pos );
                        }
		bitBlt( &myself, coords( rank, file ), &buffer, buffer.rect(), Qt::CopyROP ,false);
		return;
	}

	/*
			Cache miss
			Draw the pixmap
	*/
	if( color( rank, file ) )
		buffer = Resource->SquareLight;
	else
		buffer = Resource->SquareDark;

	switch( Logic->current[pos].Note )
	{
		case NOTE_HIGHLIGHT:	// Fall Through
		case NOTE_SELECT:
                        {
                        QPixmap *px = &Resource->HighlightSelect;
                        if (px->mask() == 0L)
           			qDebug("Null mask");
                        //px->setMask(px->createHeuristicMask());
			bitBlt( &buffer, 0, 0,px, 0, 0, -1, -1, Qt::CopyROP, FALSE);
			break;
                        }
		case NOTE_MOVE:				// Fall Through
		case NOTE_CASTLE:			// Fall Through
		case NOTE_PAWN_DOUBLE:
			bitBlt( &buffer, 0, 0, &Resource->HighlightMove, 0, 0, -1, -1, Qt::CopyROP, FALSE);
			break;
		case NOTE_ATTACK:			// Fall Through
		case NOTE_ENPASSANT:
			bitBlt( &buffer, 0, 0, &Resource->HighlightAttack, 0, 0, -1, -1, Qt::CopyROP, FALSE);
			break;
		default:
		break;
	}
	if( ( Logic->current[pos].ManPtr != Null ) && ( dragging != pos ) )
	{
  	switch( Logic->chessman[Logic->current[pos].ManPtr].Type )
  	{
  		case King:
  			if( Logic->chessman[ Logic->current[pos].ManPtr ].Army == WHITE ) {
   					QPixmap *px = &Resource->WhiteKing;
                                        //px->setMask(px->createHeuristicMask());
                                        bitBlt( &buffer, 0, 0, px, 0, 0, -1, -1, Qt::CopyROP, FALSE);
                                        
                                        
                                     }
  			else
   					bitBlt( &buffer, 0, 0, &Resource->BlackKing, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			break;
  		case Queen:
  			if( Logic->chessman[ Logic->current[pos].ManPtr ].Army == WHITE )
   					bitBlt( &buffer, 0, 0, &Resource->WhiteQueen, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			else
   					bitBlt( &buffer, 0, 0, &Resource->BlackQueen, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			break;
  		case Bishop:
  			if( Logic->chessman[ Logic->current[pos].ManPtr ].Army == WHITE )
   					bitBlt( &buffer, 0, 0, &Resource->WhiteBishop, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			else
   					bitBlt( &buffer, 0, 0, &Resource->BlackBishop, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			break;
  		case Knight:
  			if( Logic->chessman[ Logic->current[pos].ManPtr ].Army == WHITE )
   					bitBlt( &buffer, 0, 0, &Resource->WhiteKnight, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			else
   					bitBlt( &buffer, 0, 0, &Resource->BlackKnight, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			break;
  		case Rook:
  			if( Logic->chessman[ Logic->current[pos].ManPtr ].Army == WHITE )
   					bitBlt( &buffer, 0, 0, &Resource->WhiteRook, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			else
   					bitBlt( &buffer, 0, 0, &Resource->BlackRook, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			break;
  		case Pawn:
  			if( Logic->chessman[ Logic->current[pos].ManPtr ].Army == WHITE )
   					bitBlt( &buffer, 0, 0, &Resource->WhitePawn, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			else
   					bitBlt( &buffer, 0, 0, &Resource->BlackPawn, 0, 0, -1, -1, Qt::CopyROP, FALSE);
  			break;
  		default:
  			break;
  	}
	}
	if( Resource->OPTION_High_Quality )
	{
  
		tempImage = buffer;
		buffer = tempImage.smoothScale( Resource->ThemeSize, Resource->ThemeSize );
	}
	/* Now add this pixmap to the cache */
	cache->add( cacheName, buffer );
	/*  */
	if( Resource->OPTION_Show_Coord )
		drawCoords( &buffer, pos );
	bitBlt( &myself, coords( rank, file ), &buffer, buffer.rect(), Qt::CopyROP );
	
}
///////////////////////////////////////
//
//	board::drawCoords
//
///////////////////////////////////////
void board::drawCoords( QPixmap *pic, int pos )
{
//pic->setMask(pic->createHeuristicMask());
	QPainter painter;
	QString letter;
	int targetRank, targetFile;
	bool orientation( Resource->OPTION_Board_Orientation );

	//if( localArmy != WHITE ) orientation = !orientation;
	if( flip ) orientation = !orientation;
 	if( orientation == 0 )
 	{
 		targetRank = 0;
 		targetFile = 0;
 	}
 	else
 	{
 		targetRank = 7;
 		targetFile = 7;
 	}
 	if( ( pos / 8 ) == targetRank )
 	{
		letter = QString( QString("abcdefgh").at( pos % 8 ) );
 		painter.begin( pic );
 		painter.setFont( Resource->FONT_Standard );
 		painter.setPen( Resource->COLOR_Notation_Shadow );
 		painter.drawText( 3, 3, Resource->ThemeSize - 4, Resource->ThemeSize - 4,
 												Qt::AlignRight | Qt::AlignBottom, letter );
 		painter.setPen( Resource->COLOR_Notation );
 		painter.drawText( 2, 2, Resource->ThemeSize - 4, Resource->ThemeSize - 4,
 												Qt::AlignRight | Qt::AlignBottom, letter );
 		painter.end();
 	}
 	if( ( pos % 8 ) == targetFile )
 	{
		letter = QString( QString("12345678").at( pos / 8 ) );
 		painter.begin( pic );
 		painter.setFont( Resource->FONT_Standard );
 		painter.setPen( Resource->COLOR_Black );
 		painter.drawText( 3, 3, Resource->ThemeSize - 4, Resource->ThemeSize - 4,
 												Qt::AlignLeft | Qt::AlignTop, letter );
 		painter.setPen( Resource->COLOR_White );
 		painter.drawText( 2, 2, Resource->ThemeSize - 4, Resource->ThemeSize - 4,
 												Qt::AlignLeft | Qt::AlignTop, letter );
 		painter.end();
 	}
}///////////////////////////////////////
//
//	board::drawDrag
//
///////////////////////////////////////
void board::drawDrag( void )
{
	if( !isNew )
	{
		restoreDrag();
		if( Logic->current[dragging].Note != NOTE_SELECT )
		{
			Logic->current[dragging].Note = NOTE_SELECT;
			drawPosition( dragging );
		}
	}
	bitBlt( &oldDragPix, QPoint( 0, 0 ), &myself, QRect( currentPoint, dragPix.size() ), Qt::CopyROP,false );
	bitBlt( &myself, currentPoint, &dragPix, dragPix.rect(), Qt::CopyROP,false );
}
///////////////////////////////////////
//
//	board::setPaused
//
///////////////////////////////////////
void board::setPaused( const bool state )
{
	paused = state;
	redrawAll();
}
///////////////////////////////////////
//
//	board::setLocalArmy
//
///////////////////////////////////////
void board::setLocalArmy( const bool state )
{
	if( state == localArmy ) return;
	localArmy = state;
	redrawAll();
}
///////////////////////////////////////
//
//	board::flipBoard
//
///////////////////////////////////////
void board::flipBoard( void )
{
	flip = !flip;
	redrawAll();
}
///////////////////////////////////////
//
//	board::color
//
///////////////////////////////////////
bool board::color( const int rank, const int file )
{
	return abs( ( rank % 2 ) - ( file % 2 ) );
}


///////////////////////////////////////
//
//	board::mouseReleaseEvent
//
///////////////////////////////////////
void board::mouseReleaseEvent( QMouseEvent *event )
{
	int tmp;
	if( dragging != Null )
	{
		tmp = dragging;
		dragging = Null;
		restoreDrag();
		drawPosition( tmp );
		commit();
	}
	if(event->button() == Qt::LeftButton)
	{
		if (!inMiddle)
                  emit LeftClick( position( event->pos() ) );
	}
	if(event->button() == Qt::RightButton)
	{
		emit RightClick( position( event->pos() ) );
	}
     if (inMiddle)
       inMiddle=false;
}
///////////////////////////////////////
//
//	board::mousePressEvent
//
///////////////////////////////////////
void board::mousePressEvent( QMouseEvent *event )
{
	pressPoint = event->pos();
}

void board::mouseDoubleClickEvent (QMouseEvent *event)
{
qDebug("Board::mouseDoubleClickEvent");
inMiddle = true;
emit RightClick( position( event->pos() ) );

}
///////////////////////////////////////
//
//	board::mouseMoveEvent
//
///////////////////////////////////////
void board::mouseMoveEvent( QMouseEvent *event )
{

	QImage tempImage;

	if( dragging == Null )
	{
		if( event->state() & Qt::LeftButton )
		{
			if( abs( pressPoint.x() - event->pos().x() ) + abs( pressPoint.y() - event->pos().y() ) > 7 )
			{
  			/* Begin dragging a piece */
  			emit LeftClick( position( event->pos() ) ); // Tell match that we just clicked on this piece
  			dragging = position( event->pos() );
  			if( Logic->current[dragging].Note != NOTE_SELECT )
  			{ // The selection didn't take.. back out.
  				dragging = Null;
  				drawPosition( position( event->pos() ) );
  				commit();
  				return;
  			}
				drawPosition( dragging );
  			oldDragPos = event->pos();
				/* Get the piece image and store it in dragPix */
				dragPix = getChessman( dragging );
                                
//       	if( Resource->OPTION_High_Quality )
//       	{
//         	
//       		tempImage = dragPix;
//       		dragPix = tempImage.smoothScale( Resource->ThemeSize, Resource->ThemeSize );
// //                      dragPix.setMask(dragPix.createHeuristicMask());
//       	}
  			oldDragPix.resize( dragPix.size() );
  			oldDragPix.fill();
  			isNew = TRUE;
			}
			else
				/* Not enough dragging */
				return;
		} // End ( event->state() & Qt::LeftButton )
		else
			return; /* No dragging. Most events should end up here */
	} // End ( dragging == Null )
	int halfSize = Resource->ThemeSize >> 1;
	currentPoint.setX( event->x() - halfSize );
	currentPoint.setY( event->y() - halfSize );
	commit();
	oldDragPos = currentPoint;
	isNew = FALSE;
}
///////////////////////////////////////
//
//	board::restoreDrag
//
///////////////////////////////////////
void board::restoreDrag( void )
{
	bitBlt( &myself, oldDragPos, &oldDragPix, oldDragPix.rect(), Qt::CopyROP,false );
}
///////////////////////////////////////
//
//	board::getChessman
//
///////////////////////////////////////
QPixmap board::getChessman( const int pos )
{
qDebug("board::getChessman");
	QPixmap tmp;
	char type, army;

	if( pos != premoveTo )
	{
		type = Logic->chessman[Logic->current[pos].ManPtr].Type;
		army = Logic->chessman[Logic->current[pos].ManPtr].Army;
	}
	else
	{
		type = Logic->chessman[Logic->current[premoveFrom].ManPtr].Type;
		army = Logic->chessman[Logic->current[premoveFrom].ManPtr].Army;
	}
	switch( type )
	{
		case King:
                
			if( army == WHITE )
				tmp = Resource->WhiteKing;
			else
				tmp = Resource->BlackKing;
			break;
		case Queen:
			if( army == WHITE )
				tmp = Resource->WhiteQueen;
			else
				tmp = Resource->BlackQueen;
			break;
		case Bishop:
			if( army == WHITE )
				tmp = Resource->WhiteBishop;
			else
				tmp = Resource->BlackBishop;
			break;
		case Knight:
			if( army == WHITE )
				tmp = Resource->WhiteKnight;
			else
				tmp = Resource->BlackKnight;
			break;
		case Rook:
			if( army == WHITE )
				tmp = Resource->WhiteRook;
			else
				tmp = Resource->BlackRook;
			break;
		case Pawn:
			if( army == WHITE )
				tmp = Resource->WhitePawn;
			else
				tmp = Resource->BlackPawn;
			break;
		default:
			break;
	}
	if( ( pos == premoveFrom ) || ( pos == premoveTo ) ) {
        kdDebug() << "( pos == premoveFrom ) || ( pos == premoveTo ) )";
//        KIconEffect::semiTransparent( tmp );
        
        }
        return tmp;
}
///////////////////////////////////////
//
//	board::setPremovePositions
//
///////////////////////////////////////
void board::setPremovePositions( const int posF, const int posT )
{
	premoveFrom = posF;
	premoveTo = posT;
	if( ( posF != Null ) && ( posT != Null ) )
	{
		Logic->current[posF].Note = NOTE_NONE;
		Logic->current[posT].Note = NOTE_NONE;
		drawPosition( posF );
		drawPosition( posT );
		commit();
	}
}
