/***************************************************************************
                          board.h  -  description
                             -------------------
    begin                : Sat Sep 29 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BOARD_H
#define BOARD_H

#include <qwidget.h>
#include "resource.h"
#include "logic.h"

/**
  *@author Troy Corbin Jr.
  */

class board : public QWidget
{
   Q_OBJECT
	public:
											board(QWidget *parent=0, const char *name=0, resource *Rsrc=0, logic *Lgc=0);
											~board();
		void							mouseReleaseEvent( QMouseEvent *event );
		void							mousePressEvent( QMouseEvent *event );
		void							mouseMoveEvent( QMouseEvent *event );
                void							mouseDoubleClickEvent(QMouseEvent *event);
		void							setPremovePositions( const int posF, const int posT );
                
	public slots:
		void							flipBoard( void );
		void 							ResizeBoard( void );
		void							redrawAll( void );
		void							commit( void );
		void 							drawPosition( const int rank, const int file );
		void 							drawPosition( const int pos );
		void							setPaused( const bool );
		void							setLocalArmy( const bool );
//                 void							redrawLights();
	signals:
		void							LeftClick( int );
		void							RightClick( int );
	protected:
		void							drawCoords( QPixmap *pic, int pos );
		QPoint						coords( const int, const int);
		int								position( const QPoint );
		bool							color( const int, const int );
		void							drawDrag( void );
		void							restoreDrag( void );
		QPixmap						getChessman( const int pos );
	private:
		bool							paused;
		bool							localArmy;
		bool							flip;
		resource					*Resource;
		logic							*Logic;
		KnightsPixCache		*cache;
		QPixmap						myself;
		bool orientation;
		/* This stuff is used for drag&drop chessmen */
		bool							isNew;
		int								dragging;
		QPoint						oldDragPos;
		QPoint						pressPoint;
		QPoint						currentPoint;
		QPixmap						dragPix;
		QPixmap						oldDragPix;
		/* Premove Positions */
		int								premoveFrom;
		int								premoveTo;
};

#endif
