/***************************************************************************
                          core.cpp  -  description
                             -------------------
    begin                : Tue Oct 2 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <assert.h>
#include <qpushbutton.h>
#include <qwidget.h>
#include <qlistbox.h>
#include <kfiledialog.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qpe/fileselector.h>
#include "core.h"
#include "qinputdialog.h"
#include <unistd.h>
#include "qinputdialog.h"

///////////////////////////////////////
//
//	core::core
//
///////////////////////////////////////
core::core(QWidget *parent, const char *name, resource *Rsrc ) : QWidget(parent,name)
{
	Resource = Rsrc;
	listPGN = NULL;
	ButtonGroup = new thinbuttons( this, "ButtonGroup", Resource );
	ButtonGroup->show();
	connect( ButtonGroup, SIGNAL( clicked(int) ), this, SLOT( setMatch(int) ) );

	connect( &CoreClock, SIGNAL( timeout() ), this, SLOT( CoreLoop() ) );
	CoreClock.start( 100, FALSE );
	ID_Current = Null;
	ID_Last_ICS_Match = Null;

	/* Init the lists */
	L_Match.setAutoDelete( TRUE );
	L_Engine.setAutoDelete( TRUE );
	//L_Server.setAutoDelete( TRUE );
	resize();
}
///////////////////////////////////////
//
//	core::~core
//
///////////////////////////////////////
core::~core()
{
	CoreClock.stop();
	L_Engine.clear();
	//L_Server.clear();
	L_Match.clear();
	delete ButtonGroup;
}
///////////////////////////////////////
//
//	core::CoreLoop
//
///////////////////////////////////////
void core::CoreLoop( void )
{
	
	engineio *engine_Ptr( NULL );
//	internetio *server_Ptr( NULL );
	match *match_Ptr( L_Match.first() );
	int CMD;
	unsigned int IndexM, IndexE, IndexS;

	/*
			Check all Matches
	*/
	for( IndexM = 0; IndexM < L_Match.count(); IndexM++ )
	{
		match_Ptr = L_Match.at(IndexM);
		match_Ptr->tick();
	}
	/*
			Check Engines
	*/
	for( IndexE = 0; IndexE < L_Engine.count(); IndexE++ )
	{
		engine_Ptr = L_Engine.at(IndexE);
   	CMD = engine_Ptr->Poll();
		if( engine_Ptr->InitTimer && engine_Ptr->Check )
			initEngines(TRUE);
		if( CMD == CMD_None ) continue;
		findMatch( engine_Ptr->matchID() );
		/* Process Offer Draw */
		if( CMD == CMD_Offer_Draw )
			L_Match.current()->setDrawOffered( engine_Ptr->Army );
		/* Process Hint */
		if( CMD == CMD_Hint ) {
                	kdDebug() << "Hint" << ( engine_Ptr->Move.SAN ) << endl;
			QMessageBox::information( this,  i18n("Hint:"),i18n( "%1 suggests this move:\n%2" ).arg( engine_Ptr->FEATURE_MyName ).arg( engine_Ptr->Move.SAN ) );
                        }
		/* Process Tell User */
		if( CMD == CMD_Tell_User )
			QMessageBox::information( this, QString( engine_Ptr->FEATURE_MyName), i18n( "%1 tells you:\n%2" ).arg( engine_Ptr->FEATURE_MyName ).arg( engine_Ptr->Inbound ) );
		/* Process Tell User Error */
		if( CMD == CMD_Tell_User_Error )
			QMessageBox::information( this, QString( engine_Ptr->FEATURE_MyName ), i18n( "%1 tells you:\n%2" ).arg( engine_Ptr->FEATURE_MyName ).arg( engine_Ptr->Inbound ) );
		/* Process Out Of Book */
		if( CMD == CMD_Out_Of_Book )
		{
//			Fix this by 0.6
//			engine[enginePtr - 1]->sendCMD( CMD_Check_Book );
		}
		/* Process Illegal move commands */
  	if( ( CMD == CMD_Illegal ) &&
				( L_Match.current()->onMove() != Null ) &&
				( !engine_Ptr->InitTimer ) )
		{
			L_Match.current()->retract();
			emit setNotation();
		}
		/* Move */
  	if( CMD == CMD_Move )
  		L_Match.current()->slot_Move( engine_Ptr->Move );
	}
 	/*
			Check Servers
	*/
// 	for( IndexS = 0; IndexS < L_Server.count(); IndexS++ )
// 	{
// 	 	server_Ptr = L_Server.at(IndexS);
//  		CMD = server_Ptr->Poll();
// 		/* Process Raw Input ( Repeat as needed ) */
// 		while( CMD == CMD_Raw_Input ) CMD = server_Ptr->Poll();
// 		/* Move */
// 		if( CMD == CMD_Move )
// 		{
// 			findMatch( server_Ptr->matchID() );
// 			L_Match.current()->setBoard( server_Ptr->Pos );
// 		}
// 		/* Has the Game's Status Changed? */
// 		if( server_Ptr->extendedData )
// 		{
// 			bool army = WHITE;
// 			if( CMD == CMD_Result_Black )
// 				army = BLACK; // If not result Black, then the result is either White or it doesn't matter.
// 			findMatch( server_Ptr->matchID() );
// 			switch( server_Ptr->extendedData )
// 			{
// 				case 1:
// 				case 3: // Someone resigned.
// 					L_Match.current()->endMatch( CMD_Resign, army );
// 					break;
// 				case 2: // Flag was called.
// 					L_Match.current()->endMatch( CMD_Call_Flag, army );
// 					break;
// 				case 4: // Draw game.
// 					L_Match.current()->endMatch( CMD_Result_Draw, army );
// 					break;
// 				case 5: // Lost contact.
// 					L_Match.current()->endMatch( CMD_Lost_Contact, army );
// 					break;
// 				case 6: // Checkmate
// 					if( army == WHITE )
// 						L_Match.current()->endMatch( CMD_Result_White, army );
// 					else
// 						L_Match.current()->endMatch( CMD_Result_Black, army );
// 				default:
// 					break;
// 			}
// 		}
// 		/* Process Illegal move commands */
//   	if( CMD == CMD_Illegal )
// 		{
// 			findMatch( server_Ptr->matchID() );
// 			L_Match.current()->retract();
// 			emit setNotation();
// 		}
// 		/* Reading movelist */
// 		if( CMD == CMD_Read_Movelist )
// 		{
// 			findMatch( server_Ptr->matchID() );
// 			L_Match.current()->readMovelist( server_Ptr->Inbound );
// 		}
//  	}

}
///////////////////////////////////////
//
//	core::addServer
//
///////////////////////////////////////
/*netconcomm* core::addServer( void )
{
// 	internetio *server_Ptr( NULL );
// 
// 	server_Ptr = new internetio( this, "Server", Resource );
// 	L_Server.append( server_Ptr );
// 	connect( server_Ptr, SIGNAL( newGame( const char ) ), this, SLOT( addOnlineMatch( const char ) ) );
// 	connect( server_Ptr, SIGNAL( shutdown( void ) ), this, SLOT( serverSelfDestruct( void ) ) );
// 	return server_Ptr->NetConComm;
}*/
///////////////////////////////////////
//
//	core::removeServer
//
///////////////////////////////////////
void core::removeServer( void )
{
// 	internetio *server_Ptr( NULL );
// 	int loop, mainLoop;
// 
// 	for( mainLoop = 0; mainLoop < (signed)L_Server.count(); mainLoop++ )
// 	{
// 		server_Ptr = L_Server.at( mainLoop );
// 		for( loop = 0; loop < (signed)L_Match.count(); loop++ )
// 		{
// 			findMatch( loop );
// 			if( !server_Ptr->matchID( L_Match.current()->matchID() ) ) continue;
// 			clearMatch();
// 			loop = -1;
// 		}
// 		L_Server.remove();
// 	}
}
///////////////////////////////////////
//
//	core::serverSelfDestruct
//
///////////////////////////////////////
void core::serverSelfDestruct( void )
{
// 	KMessageBox::sorry( this,
// 								i18n( "Knights could not log into this server. Please check your username and password before trying again." ),
// 								i18n( "Server Login Failed" ) );
// 	emit serverDestroyed();
// 	removeServer();
}
///////////////////////////////////////
//
//	core::addOnlineMatch
//
///////////////////////////////////////
void core::addOnlineMatch( const char Army )
{
// 	if( ( ID_Last_ICS_Match != Null ) && ( Resource->OPTION_Auto_Close_Last_ICS ) )
// 	{
// 		if( findMatch( ID_Last_ICS_Match ) )
// 		{
// 			setMatch( ID_Last_ICS_Match );
// 			clearMatch();
// 		}
// 	}
// 	addMatch( MENU_VS_TCPIP );
// 	if( Army == BOTH ) L_Match.current()->setInput( PLAYERLOCAL, PLAYERLOCAL );
// 	if( Army == Null ) L_Match.current()->setInput( PLAYERTCP, PLAYERTCP );
// 	if( Army == WHITE )
// 	{
// 		L_Match.current()->setInput( PLAYERLOCAL, PLAYERTCP );
// 		ID_Last_ICS_Match = ID_Current;
// 	}
// 	if( Army == BLACK )
// 	{
// 		L_Match.current()->setInput( PLAYERTCP, PLAYERLOCAL );
// 		ID_Last_ICS_Match = ID_Current;
// 	}
// 	L_Server.current()->addMatchID( L_Match.current()->matchID() );
}
///////////////////////////////////////
//
//	core::addMatch
//
///////////////////////////////////////
void core::addMatch( int type )
{
qDebug("core::addMatch start");
	QPushButton *newButton;
	QString name;
	match *match_Ptr( NULL );

	/* Create the match navigation button */
	newButton = new QPushButton( ButtonGroup, "MatchSelector" );
	newButton->setToggleButton( TRUE );
	newButton->setText( QString("Match") );
	newButton->show();
	/* Create the match */
        qDebug("addmatch 1");
	name = QString("match %1").arg( ButtonGroup->id( newButton ) );
	switch( type )
	{
		case MENU_SOLITARE:
                	qDebug("addmatch 2");
			match_Ptr = new match( this, name, Resource );
			qDebug("addmatch 3");
                        match_Ptr->setMatchID( ButtonGroup->id( newButton ) );
			match_Ptr->setInput( PLAYERLOCAL, PLAYERLOCAL );
			L_Match.append( match_Ptr );
                        
			break;
		case MENU_VS_PC:
			match_Ptr = new match( this, name, Resource );
			match_Ptr->setMatchID( ButtonGroup->id( newButton ) );
			if( Resource->OPTION_Play_Black == TRUE ) match_Ptr->setInput( PLAYERPC, PLAYERLOCAL );
			else match_Ptr->setInput( PLAYERLOCAL, PLAYERPC );
			L_Match.append( match_Ptr );
			initEngines();
			break;
		case MENU_VS_TCPIP:
			match_Ptr = new match( this, name, Resource );
			match_Ptr->setMatchID( ButtonGroup->id( newButton ) );
			L_Match.append( match_Ptr );
			break;
		case MENU_VS_EMAIL:
			match_Ptr = new match( this, name, Resource );
			match_Ptr->setMatchID( ButtonGroup->id( newButton ) );
			match_Ptr->setInput( PLAYERLOCAL, PLAYERLOCAL );
			L_Match.append( match_Ptr );
			break;
		case MENU_LOAD:
			match_Ptr = new match( this, name, Resource );
			match_Ptr->setMatchID( ButtonGroup->id( newButton ) );
			match_Ptr->setInput( PLAYERLOAD, PLAYERLOAD );
			L_Match.append( match_Ptr );
			break;
		case MENU_PC_VS_PC:
			match_Ptr = new match( this, name, Resource );
			match_Ptr->setMatchID( ButtonGroup->id( newButton ) );
			match_Ptr->setInput( PLAYERPC, PLAYERPC );
			L_Match.append( match_Ptr );
			initEngines();
			break;
		default:
			kdDebug() << "core::addMatch: Tried starting a match with an unknown type." << endl;
			return;
	}
	connect( match_Ptr, SIGNAL( moved( const ChessMove, const int ) ), this, SLOT( relayMove( const ChessMove, const int ) ) );
	connect( match_Ptr, SIGNAL( sendCMD( const int, const int, const QString &) ), this, SLOT( relayCMD( const int, const int, const QString & ) ) );
	setMatch( match_Ptr->matchID() );
	emit requestResize();
	setActiveWindow();
        qDebug("core::addMatch end");
}
///////////////////////////////////////
//
//	core::findMatch
//
///////////////////////////////////////
bool core::findMatch( const int ID )
{
  unsigned int Index;
	
	for( Index = 0; Index < L_Match.count(); Index++ )
	{
		L_Match.at( Index );
		if( L_Match.current()->matchID() == ID ) return TRUE;
	}
	return FALSE;
}
///////////////////////////////////////
//
//	core::setMatch
//
///////////////////////////////////////
void core::setMatch( int ID )
{
	setMatch( ID, TRUE );
}
void core::setMatch( int ID, const bool decouple )
{
     kdDebug() << "core::setMatch" << endl;
	/*
			Disconnect the current match from the display
	*/
	findMatch( ID_Current );
	if( decouple )
	{
		L_Match.current()->setCurrent( FALSE );
		disconnect( L_Match.current(), SIGNAL( setClocks() ), this, SLOT( slot_setClocks() ) );
		disconnect( L_Match.current(), SIGNAL( setStatusBar( const int, const QString& ) ), this, SLOT( slot_setStatusBar( const int, const QString& ) ) );
		disconnect( L_Match.current(), SIGNAL( setNotation() ), this, SLOT( slot_setNotation() ) );
		L_Match.current()->setVisibility( FALSE );
	}
	/*
			Now display the new match, and connect it's signals to the GUI
	*/
	ID_Current = ID;
	findMatch( ID );
	L_Match.current()->setVisibility( TRUE );
	ButtonGroup->setButton( ID );
	connect( L_Match.current(), SIGNAL( setClocks() ), this, SLOT( slot_setClocks() ) );
	connect( L_Match.current(), SIGNAL( setStatusBar( const int, const QString& ) ), this, SLOT( slot_setStatusBar( const int, const QString& ) ) );
	connect( L_Match.current(), SIGNAL( setNotation() ), this, SLOT( slot_setNotation() ) );
	L_Match.current()->setCurrent( TRUE );
        
        engineio *engine_Ptr( NULL );
        for( int IndexE = 0; IndexE < L_Engine.count(); IndexE++ )
	{
		engine_Ptr = L_Engine.at(IndexE);
   	if( engine_Ptr->matchID() == ID ) {
			if (engine_Ptr->nolevels ==true)
                          emit enableDifficultyMenu(false);
                        else
                          emit enableDifficultyMenu(true);
                          
                          }
	}
        
        
        
        emit initMatch();
        	L_Match.current()->resendStatusBar();
}
///////////////////////////////////////
//
//	core::clearAll
//
///////////////////////////////////////
bool core::clearAll( void )
{
	while( L_Match.count() )
	{
		findMatch( ID_Current );
		if( clearMatch() != TRUE ) return FALSE;
	}
	return TRUE;
}
///////////////////////////////////////
//
//	core::clearMatch
//
///////////////////////////////////////
bool core::clearMatch( void )
{
	QButton *oldButton;
	int NewIndex(Null), IndexE, IndexS;

	if( modified() )
           {
		if(Resource->PromptForSaving=="Ask")
                  if( save( TRUE, FALSE ) == FALSE ) return FALSE;
                if(Resource->PromptForSaving=="Yes")
                  if( save( FALSE, FALSE ) == FALSE ) return FALSE;
                
           }

	/* Get ID of previous match so we can display it */
	findMatch( ID_Current );
	if( L_Match.next() != NULL ) NewIndex = L_Match.current()->matchID();
	else
	{
		findMatch( ID_Current );
		if( L_Match.prev() != NULL ) NewIndex = L_Match.current()->matchID();
	}

	/* Remove engines in this match... */
	for( IndexE = 0; IndexE < (signed)L_Engine.count(); IndexE++ )
	{
		L_Engine.at( IndexE );
   	if( L_Engine.current()->matchID() == ID_Current )
		{
			L_Engine.remove();
			IndexE = -1;
		}
	}
 	/* Check Servers */
 	#ifdef NOINTERNET
        for( IndexS = 0; IndexS < (signed)L_Server.count(); IndexS++ )
 	{
 	 	L_Server.at(IndexS);
    	if( L_Server.current()->matchID( ID_Current ) )
 			L_Server.current()->removeMatchID( ID_Current );
 	}
        #endif

	/* ...and remove the match */
	findMatch( ID_Current );
	if( L_Match.remove() == FALSE )
		kdDebug() << "core::clearMatch: Can not remove current match from QList" << endl;

	/* Be sure to remove the button too. */
	oldButton = ButtonGroup->find( ID_Current );
	ButtonGroup->remove( oldButton );
	delete oldButton;

	/* Set the new current match */
	if( NewIndex != Null ) setMatch( NewIndex, FALSE );
	else emit initMatch();
	resize();
	return TRUE;
}
///////////////////////////////////////
//
//	core::resize
//
///////////////////////////////////////
void core::resize( void )
{
 kdDebug() << "core::resize" << endl;
    unsigned int Index;
	
	for( Index = 0; Index < L_Match.count(); Index++ )
	{
		L_Match.at( Index );
		L_Match.current()->resize();
		L_Match.current()->move( 0, 0 );
	}
	if( L_Match.count() )
	{
		ButtonGroup->resize( L_Match.current()->width() );
		ButtonGroup->move( 0, L_Match.current()->height() );
		setFixedSize( L_Match.current()->width(), L_Match.current()->height() + ButtonGroup->height() );
	}
	else
	{
		ButtonGroup->resize( Resource->ThemeSize * 8 );
		ButtonGroup->move( 0, Resource->ThemeSize * 8 );
		setFixedSize( Resource->ThemeSize * 8, ( Resource->ThemeSize * 8 ) + ButtonGroup->height() );		
	}
}
///////////////////////////////////////
//
//	core::relayCMD
//
///////////////////////////////////////
void core::relayCMD( const int CMD, const int IndexM, const QString &value )
{
	match *match_Ptr( NULL );

	findMatch( IndexM );
	match_Ptr = L_Match.current();

	if( CMD == CMD_New_Players )
	{
		findMatch( match_Ptr->matchID() );
		initEngines();
	}
	sendCMD( match_Ptr->matchID(), CMD,value );
}
///////////////////////////////////////
//
//	core::sendCMD
//
///////////////////////////////////////
void core::sendCMD( const int ID, const int CMD, const QString & value )
{
         kdDebug() << "core::sendCMD.  ID: " << ID << " CMD: " << CMD << " Value: " << value;
	engineio *engine_Ptr( NULL );
	#ifdef INTERNETPLAY
        internetio *server_Ptr( NULL );
        #endif
	unsigned int IndexE, IndexS;

	/* Check Engines */
	for( IndexE = 0; IndexE < L_Engine.count(); IndexE++ )
	{
		engine_Ptr = L_Engine.at(IndexE);
   	if( engine_Ptr->matchID() == ID ) {
			kdDebug() << "inside core::sendCMD engine loop" << endl;
                        engine_Ptr->SendCMD( CMD ,value);
                        }
	}
        kdDebug() << "outside core::sendCMD engine loop" << endl;
 	/* Check Servers */
	#ifdef INTERNETPLAY
        for( IndexS = 0; IndexS < L_Server.count(); IndexS++ )
	{
	 	server_Ptr = L_Server.at(IndexS);
   	if( server_Ptr->matchID( ID ) )
			server_Ptr->sendCMD( CMD, ID,value );
 }
 #endif
}
///////////////////////////////////////
//
//	core::relayMove
//
///////////////////////////////////////
void core::relayMove( const ChessMove chessMove, const int IndexM )
{
	match *match_Ptr( NULL );
	engineio *engine_Ptr( NULL );
	#ifdef INTERNETPLAY
        internetio *server_Ptr( NULL );
        #endif
	unsigned int IndexE, IndexS;
	char inputMoving;

	findMatch( IndexM );
	match_Ptr = L_Match.current();
	if( match_Ptr->onMove() == WHITE )
		inputMoving = match_Ptr->whiteInput();
	else
		inputMoving = match_Ptr->blackInput();

	/* Check Engines */
	for( IndexE = 0; IndexE < L_Engine.count(); IndexE++ )
	{
		engine_Ptr = L_Engine.at(IndexE);
   	if( engine_Ptr->matchID() == IndexM )
		{
			if( ( inputMoving != PLAYERPC ) || ( engine_Ptr->Army != match_Ptr->onMove() ) )
			{
  			engine_Ptr->SendCMD( CMD_Your_Time, match_Ptr->centiseconds( engine_Ptr->Army ) );
	  		engine_Ptr->SendCMD( CMD_Enemy_Time, match_Ptr->centiseconds( !engine_Ptr->Army ) );
				engine_Ptr->Move = chessMove;
				engine_Ptr->SendCMD( CMD_Move );
			}
			engine_Ptr->Turn = !engine_Ptr->Turn;
		}
	}
 	/* Check Servers */
        
        #ifdef INTERNETPLAY
	for( IndexS = 0; IndexS < L_Server.count(); IndexS++ )
	{
	 	server_Ptr = L_Server.at(IndexS);
		if( server_Ptr->matchID( IndexM ) ) server_Ptr->Send( QString( chessMove.SAN ) );
 	
        }
        #endif
}
///////////////////////////////////////
//
//	core::flag
//
///////////////////////////////////////
bool core::flag( const bool Army )
{
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return FALSE;
	return L_Match.current()->flag( Army );
}
///////////////////////////////////////
//
//	core::clock
//
///////////////////////////////////////
QString core::clock( const bool Army )
{
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return QString::null;
	return L_Match.current()->clock( Army );
}
///////////////////////////////////////
//
//	core::modified
//
///////////////////////////////////////
bool core::modified( void )
{
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return FALSE;
	return L_Match.current()->modified();
}
///////////////////////////////////////
//
//	core::paused
//
///////////////////////////////////////
bool core::paused( void )
{
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return FALSE;
	return L_Match.current()->paused();
}
///////////////////////////////////////
//
//	core::isOnline
//
///////////////////////////////////////
bool core::isOnline( void )
{
	#ifdef INTERNETPLAY
        if( L_Server.count() ) return TRUE;
	return FALSE;
        #endif
        return false;
}
///////////////////////////////////////
//
//	core::onMove
//
///////////////////////////////////////
char core::onMove( void )
{
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return Null;
	return L_Match.current()->onMove();
}
///////////////////////////////////////
//
//	core::caption
//
///////////////////////////////////////
QString core::caption( void )
{
	QButton *Button;
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return QString::null;
	Button = ButtonGroup->find( ID_Current );
	if( Button != NULL ) Button->setText( L_Match.current()->caption() );
	return L_Match.current()->caption();
}
///////////////////////////////////////
//
//	core::setModified
//
///////////////////////////////////////
void core::setModified( const bool state )
{
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return;
	L_Match.current()->setModified( state );
}
///////////////////////////////////////
//
//	core::whiteInput
//
///////////////////////////////////////
char core::whiteInput(void)
{
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return Null;
	return L_Match.current()->whiteInput();
}
///////////////////////////////////////
//
//	core::blackInput
//
///////////////////////////////////////
char core::blackInput(void)
{
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return Null;
	return L_Match.current()->blackInput();
}
///////////////////////////////////////
//
//	core::review
//
///////////////////////////////////////
void core::review( int Index )
{
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return;
	L_Match.current()->review( Index );
}
///////////////////////////////////////
//
//	core::notation
//
///////////////////////////////////////
QStringList* core::notation( void )
{
	findMatch( ID_Current );
	if( L_Match.current() == NULL ) return NULL;
	return L_Match.current()->notation();
}
///////////////////////////////////////
//
//	core::slot_setNotation
//
///////////////////////////////////////
void core::slot_setNotation( void )
{
	emit setNotation();
}
///////////////////////////////////////
//
//	core::slot_setStatusBar
//
///////////////////////////////////////
void core::slot_setStatusBar( const int ID, const QString& MSG )
{
	emit setStatusBar( ID, MSG );
}
///////////////////////////////////////
//
//	core::slot_setClocks
//
///////////////////////////////////////
void core::slot_setClocks( void )
{
	emit setClocks();
}
///////////////////////////////////////
//
//	core::listDestroyed
//
///////////////////////////////////////
void core::listDestroyed( void )
{
	qDebug("core::listDestroyed");
	listPGN = NULL;
}
///////////////////////////////////////
//
//	core::swapInput
//
///////////////////////////////////////
void core::swapInput( void )
{
	if( Resource->OPTION_Play_Black == TRUE )
	{
		if( ( blackInput() == PLAYERPC ) && ( whiteInput() == PLAYERLOCAL ) )
		{
			findMatch( ID_Current );
			L_Match.current()->setInput( PLAYERPC, PLAYERLOCAL );
			sendCMD( ID_Current, CMD_Play_White );
		}
	}
	if( Resource->OPTION_Play_Black == FALSE )
	{
		if( ( blackInput() == PLAYERLOCAL ) && ( whiteInput() == PLAYERPC ) )
		{
			findMatch( ID_Current );
			L_Match.current()->setInput( PLAYERLOCAL, PLAYERPC );
			sendCMD( ID_Current, CMD_Play_Black );
		}
	}
}
///////////////////////////////////////
//
//	core::submitMessage
//
///////////////////////////////////////
void core::submitMessage( const QString& MSG )
{
	#ifdef INTERNETPLAY
        if( L_Server.current() == NULL ) return;
	L_Server.current()->Send( MSG );
        #endif
}
///////////////////////////////////////
//
//	core::load
//
///////////////////////////////////////
bool core::load( void )
{
//const QString fileFilter( ".pgn" );
// 	QString URL;
// 	//URL = KFileDialog::getOpenFileName( QString::null, QString( "*" + fileFilter ), this, i18n( "Load Game..." ) );
// 	URL = KFileDialog::getOpenFileName( QString::null, QString( "*" + fileFilter ) );
//         if( URL.isEmpty() ) return FALSE;
// //	return load( URL );
//         return true;
kdDebug() << "core::load" << endl;
fileSelectorDialog = new QDialog(this,"dialog",true);
fileSelectorDialog->setMinimumSize(190,240);
fileSelectorDialog->setCaption("Load Match");
QVBoxLayout *l = new QVBoxLayout(fileSelectorDialog,6,6);

FileSelector *selector = new FileSelector("application/x-chess-pgn",fileSelectorDialog,"selector",false,true);
l->addWidget(selector);

if (fileSelectorDialog->exec()==QDialog::Accepted)
{
   
   const DocLnk *d = selector->selected();
   load(*d);
   delete d;
}
delete fileSelectorDialog;






return true;

}
void core::load( const DocLnk &lnk )
{
	kdDebug() << "core::load (const DocLnk &lnk)" << endl;
	if( listPGN == NULL )
	{
		listPGNDialog = new QDialog(0,"listPGNDialog",true);
                listPGNDialog->setMinimumSize(190,240);
                listPGNDialog->setCaption("Select match");
                QVBoxLayout *l = new QVBoxLayout(listPGNDialog,6,6);
                
                listPGN = new list_pgn(listPGNDialog);
                l->addWidget(listPGN);
                listPGN->setURL( lnk.file() );
//                 sleep(1);
//                 if (listPGN->listView->childCount()==1) { 
//                 kdDebug() << "only one child" << endl;
//                 load(1);
//                 }
//                 else {
                if (listPGNDialog->exec()==QDialog::Accepted) {
                   loadFromDialog();
                   listDestroyed();
                 } 
                 // }		//connect( listPGN, SIGNAL( selected( const int ) ), this, SLOT( load( const int ) ) );
                 //connect(listPGNDialog,SIGNAL(accept()),this,SLOT(loadFromDialog()));
 		//connect( listPGN, SIGNAL( destroyed() ), this, SLOT( listDestroyed() ) );
	}
	
//return true;
}

void core::loadFromDialog()
{
kdDebug() << "core::loadFromDialog" << endl;
QListViewItem *item;
item = listPGN->listView->currentItem();
load((item->text(5)).toInt());
}

bool core::load( const int pos )
{
        qDebug("core::load(const int pos");
        listPGNDialog->hide();
	if( listPGN == NULL ) return FALSE;
	addMatch( MENU_LOAD );
	return L_Match.current()->load( listPGN->URL(), pos );
}
///////////////////////////////////////
//
//	core::save
//
///////////////////////////////////////
bool core::save( const bool Prompt, const bool SaveAs )
{
  if (Prompt) {
    
     switch( QMessageBox::information( this, "Knights",
                                      "Do you wish to save your game?\n",
                                      "&Save", "&Don't Save", "&Cancel",
                                      0,      // Enter == button 0
                                      2 ) ) { // Escape == button 2
    case 0: // Save clicked, Alt-S or Enter pressed.
         dosave(SaveAs);
         break;
    case 1: // Don't Save clicked or Alt-D pressed
        // don't save but exit
        break;
    case 2: // Cancel clicked, Alt-C or Escape pressed
        return false;
        // don't exit
        break;
  }
  return true;
  } // if (Prompt)
     
  if (!Prompt)
  {
  bool a = dosave(SaveAs);
  return a;
  
  }
}  

bool core::dosave(const bool SaveAs) {
        // save
        bool exit = false;
        QString URL;
        QString inputURL;

        findMatch( ID_Current );
        if( L_Match.current() == NULL )
          return FALSE;

        URL = L_Match.current()->url();
        if( ( URL.isEmpty() ) || ( SaveAs ) )
        {

          bool ok;
          while (exit==false) {
            inputURL = QInputDialog::getText("Save file?","Enter name for game:","Game 1",&ok);
            if (!ok) {
              kdDebug() << "User cancelled" << endl;
              return false;
            }
               
            if (ok)
            {                  
              if( inputURL.isNull() )
                return false;
              else {
                QDir Dir(QDir::homeDirPath() + "/Applications/knights/");
                if (!Dir.exists());
                  Dir.mkdir(QDir::homeDirPath() + "/Applications/knights/");
                            
                URL = QDir::homeDirPath() + "/Applications/knights/" + inputURL + ".pgn";
                QFileInfo fileInfo(URL);
                if (fileInfo.exists()) {
                  QMessageBox mb( "Overwrite?",
                      "File already exists.\nOverwrite?",
                      QMessageBox::Information,
                      QMessageBox::Yes | QMessageBox::Default,
                      QMessageBox::No,QMessageBox::Cancel);
                    
                  switch (mb.exec()) {
                    case QMessageBox::Yes:
                      exit=true;
                      //save and exit
                      break;
                    case QMessageBox::Cancel:
                      //exit
                      break;
                  }
                    
          } else {
            exit=true;
          }
        } 
        if (!(URL.isEmpty()))  {
          if (L_Match.current()->save(URL) == false)
            {
              emit setStatusBar (SAVE_ERROR);
              return false;
            }
            DocLnk l(URL,false);
            l.setName(inputURL);
            setModified(false);
            l.writeLink();
         }      
       }
                      
    } 
    
}

return true;
}
        
          
          // 	if( L_Match.current()->save( URL ) == FALSE )
// 	{
// 		emit setStatusBar( SAVE_ERROR );
// 		return FALSE;
// 	}
// 	emit setStatusBar( SAVE_OK );
// 	setModified( FALSE );
// 	return TRUE;


///////////////////////////////////////
//
//	core::initEngines
//
///////////////////////////////////////
void core::initEngines( const bool revisit )
{
        kdDebug() << "core::initEngines" << endl;
//	engineio *engine_Ptr(
	engineio *engine_Ptr( NULL );
	QString engineName;

	if( revisit )
	{
		engine_Ptr = L_Engine.current();
		engine_Ptr->Check = FALSE;
		engine_Ptr->Turn = WHITE;
                 
		if( engine_Ptr->BookEngine )
		{
			/* Catch regular engines being used as book engines */
			if( engine_Ptr->FEATURE_BookEngine == 0 )
			{
				if( engine_Ptr->Army == WHITE )
					emit setStatusBar( BOOK_ERROR_2, QString( engine_Ptr->FEATURE_MyName ) );
				else
					emit setStatusBar( BOOK_ERROR_4, QString( engine_Ptr->FEATURE_MyName ) );
				return;
			}
			else
			{
				engine_Ptr->Team = TRUE;
				engine_Ptr->SendCMD( CMD_Book_Mode );
				engine_Ptr->SendCMD( CMD_NewGame );
			}
		}
		else
		{
			/* Catch book engines being used as regular engines */
			if( engine_Ptr->FEATURE_BookEngine == 2 )
			{
				if( engine_Ptr->Army == WHITE )
					emit setStatusBar( BOOK_ERROR_1, QString( engine_Ptr->FEATURE_MyName ) );
				else
					emit setStatusBar( BOOK_ERROR_3, QString( engine_Ptr->FEATURE_MyName ) );
				return;
			}
		}
		/* Set pondering */
		if( Resource->OPTION_Ponder ) engine_Ptr->SendCMD( CMD_Ponder );
		else engine_Ptr->SendCMD( CMD_No_Pondering );
		engine_Ptr->SendCMD( CMD_NewGame );
		/* Set army */
		if( L_Match.current()->loading() ) engine_Ptr->SendCMD( CMD_Listen );
		else
		{
			if( engine_Ptr->Army == WHITE ) engine_Ptr->SendCMD( CMD_Play_White );
			else engine_Ptr->SendCMD( CMD_Play_Black );
		}
		/* Fetch Name */
		findMatch( engine_Ptr->matchID() );
		engineName = engine_Ptr->FEATURE_MyName;
                               
		L_Match.current()->setName( engine_Ptr->Army, engineName );
		emit initMatch();
		return;
	}
	if( L_Match.current()->whiteInput() == PLAYERPC )
	{
		engine_Ptr = new engineio( Resource );
		engine_Ptr->setMatchID( L_Match.current()->matchID() );
		engine_Ptr->Start( ENGINE_WHITE );
		engine_Ptr->SendCMD( CMD_Init );
		L_Engine.append( engine_Ptr );
		if( Resource->OPTION_Book_White )
		{
  		engine_Ptr = new engineio( Resource );
  		engine_Ptr->setMatchID( L_Match.current()->matchID() );
  		engine_Ptr->BookEngine = TRUE;
  		engine_Ptr->Start( ENGINE_WHITE_BK );
  		engine_Ptr->SendCMD( CMD_Init );
			L_Engine.append( engine_Ptr );
		}
	}
	if( L_Match.current()->blackInput() == PLAYERPC )
	{
		engine_Ptr = new engineio( Resource );
		engine_Ptr->setMatchID( L_Match.current()->matchID() );
		engine_Ptr->Start( ENGINE_BLACK );
		engine_Ptr->SendCMD( CMD_Init );
		L_Engine.append( engine_Ptr );
		if( Resource->OPTION_Book_Black )
		{
  		engine_Ptr = new engineio( Resource );
  		engine_Ptr->setMatchID( L_Match.current()->matchID() );
  		engine_Ptr->BookEngine = TRUE;
  		engine_Ptr->Start( ENGINE_BLACK_BK );
  		engine_Ptr->SendCMD( CMD_Init );
			L_Engine.append( engine_Ptr );
		}
	}
        
        engine_Ptr = new engineio(Resource);
        engine_Ptr->setMatchID( L_Match.current()->matchID() );
                
}
///////////////////////////////////////
//
//	core::matchMenu
//
///////////////////////////////////////
void core::matchMenu( int opt )
{
	//assert(ID_Current);
	unsigned int loop;
	switch(opt)
	{
		case MENU_OFFER_DRAW:		// Fall Through
		case MENU_ACCEPT_DRAW:
			findMatch( ID_Current );
			if( whiteInput() == PLAYERLOCAL ) L_Match.current()->setDrawOffered( WHITE );
			if( blackInput() == PLAYERLOCAL ) L_Match.current()->setDrawOffered( BLACK );
			sendCMD( ID_Current, CMD_Offer_Draw );
			break;
		case MENU_REJECT_DRAW:
			sendCMD( ID_Current, CMD_Reject_Draw );
			break;
		case MENU_RESIGN:
                	kdDebug() << "MENU_RESIGN 1" << endl;
         //               assert (ID_Current);
                        assert(CMD_Resign);
			sendCMD( ID_Current, CMD_Resign );
                        kdDebug() << "MENU_RESIGN 2" << endl;
			if( whiteInput() == PLAYERLOCAL )
				L_Match.current()->endMatch( CMD_Resign, BLACK );
			if( blackInput() == PLAYERLOCAL )
				L_Match.current()->endMatch( CMD_Resign, WHITE );
			break;
		case MENU_RETRACT:
			sendCMD( ID_Current, CMD_Retract_Move );
			L_Match.current()->retract();
			L_Match.current()->retract();
			emit setNotation();
			if( onMove() == WHITE ) emit setStatusBar( WHITE_TURN );
			else  emit setStatusBar( BLACK_TURN );
			break;
		case MENU_MOVE_NOW:
			sendCMD( ID_Current, CMD_MoveNow );
			break;
		case MENU_HINT:
			sendCMD( ID_Current, CMD_Hint );
			break;
		case MENU_ORIENTATION:
			findMatch( ID_Current );
			L_Match.current()->flip();
			break;
		case MENU_PONDER:
   		if( Resource->OPTION_Ponder ) sendCMD( ID_Current, CMD_Ponder );
			else sendCMD( ID_Current, CMD_No_Pondering );
			break;
		case MENU_PAUSE:
			findMatch( ID_Current );
			L_Match.current()->setPaused( !L_Match.current()->paused() );
			if( L_Match.current()->paused() ) sendCMD( ID_Current, CMD_Pause );
			else sendCMD( ID_Current, CMD_Resume );
			break;
		case MENU_PAUSEALL:
			for( loop = 0; loop < L_Match.count(); loop++ )
			{
				findMatch( loop );
				if( ( whiteInput() == PLAYERTCP ) || ( blackInput() == PLAYERTCP ) )
					continue;
				L_Match.current()->setPaused( !L_Match.current()->paused() );
				if( L_Match.current()->paused() ) sendCMD( ID_Current, CMD_Pause );
				else sendCMD( ID_Current, CMD_Resume );
			}
			break;
		case MENU_CALL_FLAG:
			sendCMD( ID_Current, CMD_Call_Flag );
			if( whiteInput() == PLAYERLOCAL )
				L_Match.current()->endMatch( CMD_Call_Flag, WHITE );
			if( blackInput() == PLAYERLOCAL )
				L_Match.current()->endMatch( CMD_Call_Flag, BLACK );			
                                
			break;
                case MENU_WRITE_TO_ENGINE:
                {
                	bool ok;
                	QString text ;
                        text = QInputDialog::getText("","",QString::null,&ok,this);
                        if (ok && !(text.isEmpty()))
                          {
                          findMatch(ID_Current);
                          L_Match.current();
                                                  
                          }
                        
                        break;
                }
               		default:
			break;
	}
}
///////////////////////////////////////
//
//	core::resetServer
//
///////////////////////////////////////
void core::resetServer( void )
{
	#ifdef INTERNETPLAY
        internetio *server_Ptr( NULL );
	unsigned int IndexS;
	if( !L_Server.count() ) return;

	for( IndexS = 0; IndexS < L_Server.count(); IndexS++ )
	{
	 	server_Ptr = L_Server.at(IndexS);
		server_Ptr->sendCMD( CMD_Reset_Server );
	}
        #endif
}

