/***************************************************************************
                          definitions.h  -  description
                             -------------------
    begin                : Sun Jul 1 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _DEFINITIONS_H
#define _DEFINITIONS_H

#include <kdebug.h>
#include <qvaluelist.h>
#ifndef TRUE
#	define TRUE 1
#	define FALSE 0
#endif

#define IMAGE_MAX 144
#define IMAGE_MIN 32

#define CONFIG_VERSION 1
/* General */

/*
		These are used to define the game's states.
*/
const int WHITE									= 0x00;
const int BLACK									= 0x01;
const int BOTH									= 0x02;
const int PLAYERLOCAL						= 0x00;
const int PLAYERPC							= 0x01;
const int PLAYERTCP							= 0x02;
const int PLAYEREMAIL						= 0x03;
const int PLAYERLOAD						= 0x04;

const int ENGINE_WHITE					= 0x01;
const int ENGINE_WHITE_BK				= 0x02;
const int ENGINE_BLACK					= 0x04;
const int ENGINE_BLACK_BK				= 0x08;

	/* Menu Entries */

/*
		These represent the various choices found
		throughout the game's menubar.
*/

/* File Menu */
const int MENU_NEWGAME					= 0x01;
const int MENU_LOAD							= 0x02;
const int MENU_SAVE							= 0x03;
const int MENU_SAVEAS						= 0x04;
const int MENU_PRINT						= 0x05;
const int MENU_QUIT							= 0x06;
const int MENU_CLOSE						= 0x08;
const int MENU_CLOSEALL					= 0x09;
const int MENU_CONNECT					= 0x0A;
/* Options Menu */
const int MENU_PLAYWHITE				= 0x11;
const int MENU_PLAYBLACK				= 0x12;
const int MENU_BINDINGS_DIALOG	= 0x13;
const int MENU_SETTINGS_DIALOG	= 0x14;
/* New Games Menu */
const int MENU_SOLITARE					= 0x21;
const int MENU_VS_PC						= 0x22;
const int MENU_VS_TCPIP					= 0x23;
const int MENU_VS_EMAIL					= 0x24;
const int MENU_PC_VS_PC					= 0x25;
/* Match Menu */
const int MENU_RETRACT					= 0x31;
const int MENU_DRAW							= 0x32;
const int MENU_MOVE_NOW					= 0x33;
const int MENU_HINT							= 0x34;
const int MENU_PONDER						= 0x35;
const int MENU_PAUSE						= 0x36;
const int MENU_PAUSEALL					= 0x37;
const int MENU_ORIENTATION			= 0x38;
const int MENU_RESIGN						= 0x39;
const int MENU_CALL_FLAG				= 0x3A;
/* Draw Menu */
const int MENU_OFFER_DRAW				= 0x41;
const int MENU_ACCEPT_DRAW			= 0x42;
const int MENU_REJECT_DRAW			= 0x43;
const int MENU_IGNORE_DRAW			= 0x44;
/* Seek Menu */
const int MENU_SEEK							= 0x51;
const int MENU_FINGER						= 0x52;
const int MENU_TELL							= 0x53;
const int MENU_NOTIFY						= 0x54;
const int MENU_ACCEPT_MATCH			= 0x55;
const int MENU_CENSOR						= 0x56;
const int MENU_HISTORY					= 0x57;
const int MENU_ASSESS						= 0x58;
/* View Menu */
const int MENU_COPY							= 0x61;
const int MENU_SELECT_ALL				= 0x62;
const int MENU_CLEAR_ALL				= 0x63;
const int MENU_FIND							= 0x64;
const int MENU_FIND_NEXT				= 0x65;
/* Tutorial Menu */
const int MENU_OPEN_TUTORIAL		= 0x71;
const int MENU_WRITE_TO_ENGINE = 0x81;
const int MENU_DIFFICULTY = 0x82;
/* Difficulty */
const int MENU_DIFFICULTY_VERY_EASY = 0x83;
const int MENU_DIFFICULTY_EASY = 0x84;
const int MENU_DIFFICULTY_MODERATE = 0x85;
const int MENU_DIFFICULTY_HARD = 0x86;
const int MENU_DIFFICULTY_VERY_HARD = 0x87;
		/* Messages */

/*
			The following constants define messages
			viewed in the statusbar.
*/
const int READY									= 0x00000000;
const int WHITE_TURN						= 0x00000001;
const int BLACK_TURN						= 0x00000002;
const int WHITE_WIN							= 0x00000003;
const int BLACK_WIN							= 0x00000004;
const int GAME_DRAW							= 0x00000005;
const int GAME_50_MOVES					= 0x00000006;
const int WAITING								= 0x00000007;
const int ILLEGAL_MOVE					= 0x00000008;
const int NO_MOVE_WHILE_REVIEW	= 0x00000009;
const int LOAD_OK								= 0x00000010;
const int SAVE_OK								= 0x00000011;
const int PAUSED								= 0x00000012;
const int READING_FILE					= 0x00000013;
const int WHITE_DRAW_OFFER			= 0x00000014;
const int BLACK_DRAW_OFFER			= 0x00000015;
const int LOAD_ERROR						= 0x00000020;
const int SAVE_ERROR						= 0x00000021;
const int BOOK_ERROR_1					= 0x00000022;
const int BOOK_ERROR_2					= 0x00000023;
const int BOOK_ERROR_3					= 0x00000024;
const int BOOK_ERROR_4					= 0x00000025;
const int ENGINE_DIED_ERROR			= 0x00000026;
const int WHITE_RESIGN					= 0x00000027;
const int BLACK_RESIGN					= 0x00000028;
const int WHITE_FLAG						= 0x00000029;
const int BLACK_FLAG						= 0x00000030;
const int WHITE_CALL_FLAG				= 0x00000031;
const int BLACK_CALL_FLAG				= 0x00000032;
const int WHITE_CHECKMATE				= 0x00000033;
const int BLACK_CHECKMATE				= 0x00000034;
const int LOST_CONTACT					= 0x00000035;
const int COMMENT								= 0x00010000;
const int ERROR									= 0x00100000;
const int WHITE_CHECKED = 0x000000036;
const int BLACK_CHECKED = 0x000000037;

		/* Variations */
/*
			These define the variations on chess
			that can be supported.
*/

const int Type_Standard					= 0x00000001;
const int Type_WildCastle				= 0x00000002;
const int Type_NoCastle					= 0x00000004;
const int Type_FischerRandom		= 0x00000008;
const int Type_BugHouse					= 0x00000010;
const int Type_CrazyHouse				= 0x00000020;
const int Type_Losers						= 0x00000040;
const int Type_Suicide					= 0x00000080;
const int Type_Giveaway					= 0x00000100;
const int Type_TwoKings					= 0x00000200;
const int Type_Atomic						= 0x00000400;
const int Type_Kriegspiel				= 0x00000800;
const int Type_ThreeCheck				= 0x00001000;

		/* Protocols */

const int XBoard								= 0x02;
const int Knights1 							= 0x04;

		/* Commands */

/*
		These constants represent the game's desire to send the engine a command.
		SendCMD will convert them to the correct command for the current protocol.
*/

const int CMD_None							= 0x00000000;
const int CMD_Init							= 0x00000001;
const int CMD_NewGame						= 0x00000002;
const int CMD_Exit							= 0x00000003;
const int CMD_MoveNow						= 0x00000004;
const int CMD_Pause							= 0x00000005;
const int CMD_Resume						= 0x00000006;
const int CMD_Move							= 0x00000007;
const int CMD_Illegal						= 0x00000008;
const int CMD_Play_White				= 0x00000009;
const int CMD_Play_Black				= 0x00000010;
const int CMD_Result_White			= 0x00000011;
const int CMD_Result_Black			= 0x00000012;
const int CMD_Result_Draw				= 0x00000013;
const int CMD_Your_Time					= 0x00000014;
const int CMD_Enemy_Time				= 0x00000015;
const int CMD_Offer_Draw				= 0x00000016;
const int CMD_Ponder						= 0x00000017;
const int CMD_No_Pondering			= 0x00000018;
const int CMD_Book_Mode					= 0x00000019;
const int CMD_Out_Of_Book				= 0x00000020;
const int CMD_Check_Book				= 0x00000021;
const int CMD_Retract_Move			= 0x00000022;
const int CMD_Hint							= 0x00000023;
const int CMD_Listen						= 0x00000024;
const int CMD_Play							= 0x00000025;
const int CMD_Tell_User					= 0x00000026;
const int CMD_Tell_User_Error		= 0x00000027;
const int CMD_Resign						= 0x00000028;
const int CMD_Call_Flag					= 0x00000029;
const int CMD_Set_Depth					= 0x00000030;
const int CMD_Set_Board					= 0x00000031;
/* Internet specific commands */
const int CMD_Raw_Input					= 0x01000001;
const int CMD_Reject_Draw				= 0x01000002;
const int CMD_Reset_Server			= 0x01000003;
const int CMD_Read_Movelist			= 0x01000004;
const int CMD_Examine_Forward		= 0x01000005;
const int CMD_Examine_Backward	= 0x01000006;
const int CMD_Lost_Contact			= 0x01000007;
/* These commands are from Match to Core ONLY */

const int CMD_New_Players				= 0x02000001;
//Knights/E specific
const int CMD_WriteText = 0x30000001;

		/* Audio Commands */

/*
		These represent the various sounds that can be played.
*/

const char SND_NONE						= 0x00;
const char SND_SELECT					= 0x01;
const char SND_MOVE						= 0x02;
const char SND_MATCH_OVER			= 0x03;
const char SND_CHECK					= 0x04;
const char SND_FLAG						= 0x05;
const char SND_CHALLENGE			= 0x50;
const char SND_TELL						= 0x51;
const char SND_NOTIFICATION		= 0x52;
const char SND_DRAW_OFFER			= 0x53;
const char SND_SAY						= 0x54;

		/* Position Notes */
		
/*
		These represent the various NOTES that can appear on a
		given position
*/

const char NOTE_NONE						= 0x00;
const char NOTE_HIGHLIGHT				= 0x01;
const char NOTE_SELECT					= 0x02;
const char NOTE_MOVE						= 0x10;
const char NOTE_CASTLE					= 0x11;
const char NOTE_PAWN_DOUBLE			= 0x12;
const char NOTE_ATTACK					= 0x20;
const char NOTE_ENPASSANT				= 0x21;

		/* Chessmen */

/*
		These represent the various chessmen, or Types of chessmen
		found on the playing board.
*/
const char Null									= -1;
const char King									= 0;
const char Queen								= 1;
const char Bishop								= 2;
const char Knight								= 3;
const char Rook									= 4;
const char Pawn									= 5;

		/* ICS Game Modes */

const char ICS_Normal						= 0;
const char ICS_Examine					= 1;
const char ICS_Observe					= 2;

typedef struct ThemeHeader
{
	QString				name;
	QString				version;
	QString				author;
	QString				authorEmail;
	QString				authorWWW;
	QString				notes;
};

typedef struct ChessMove
{
 					char	fromFile;
 					char	fromRank;
 					char	toFile;
 					char	toRank;
 					char	Promote;
					char	ManTaken;
 					char	SAN[8];
 					char	CAN[6];
unsigned	char	NAG;
};

typedef struct Chessman
{
	bool Army;
	char Type;
	char File;
	char Rank;
};

typedef struct Position
{
	char File;
	char Rank;
	char ManPtr;
	char Note;
};

typedef struct ICSPosition
{
	bool				OnMove;
	bool				ClockTicking;
	int					WhiteTime;
	int					BlackTime;
	char				PawnPushFile;
	char				Mode;
	int					MoveCounter;
	QString			WhiteName;
	QString 		BlackName;
	QString			Board;
	QString			CAN;
	QString			SAN;
};

/* Time Control Period */
typedef struct TCP
{
	int		Moves;
	int		Seconds;
	int		Increment;
};

typedef QValueList<TCP> TCPList;

#endif
