/***************************************************************************
                          engineio.cpp  -  description
                             -------------------
    begin                : Sat Jun 30 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qregexp.h>
#include "engineio.h"

engineio::engineio( resource *Rsrc ) : QObject()
{
	Resource						= Rsrc;
	engine							= NULL;
	Log									= NULL;
	SafeToSend					= TRUE;
	FEATURE_Analyze			= TRUE;
	FEATURE_Colors			= TRUE;
	FEATURE_Draw				= TRUE;
	FEATURE_ICS					= FALSE;
	FEATURE_Name				= TRUE;
	FEATURE_Pause				= FALSE;
	FEATURE_Ping				= FALSE;
	FEATURE_PlayOther		= FALSE;
	FEATURE_Reuse				= TRUE;
	FEATURE_SetBoard		= FALSE;
	FEATURE_SAN					= FALSE;
	FEATURE_SIGINT			= TRUE;
	FEATURE_SIGTERM			= TRUE;
	FEATURE_Time				= TRUE;
	FEATURE_UserMove		= FALSE;
	FEATURE_Variants		= 0;
	FEATURE_Level				= 1;
	FEATURE_BookEngine	= 0;
	FEATURE_MyName[0]		= 0;
	CleanBuffer					= TRUE;
	SendSIGINT					= FALSE;
	Check								= TRUE;
	DelayedGo						= FALSE;
	Forced							= TRUE;
	Team								= FALSE;
	AcceptIllegal				= FALSE;
	BookEngine					= FALSE;
	Protocol						= Knights1;
}
//
engineio::~engineio()
{
	if( Log )
	{
		Log->close();
		delete Log;
	}
	engine->kill();
	delete engine;
}
///////////////////////////////////////
//
//	engineio::Start
//
///////////////////////////////////////
void engineio::Start( const int ID )
{
	QStringList args;
	unsigned int tmp;
	QString difficultyArgs;
	/* Stop accidents */
	if( engine != NULL ) return;
	engine = new KProcess();

	/* Get and parse engine config from resource */
// 	for( IT = Resource->engines.begin(); IT != Resource->engines.end(); ++IT )
// 		if( (*IT).CurrentRef & ID ) break;
// 	if( IT == Resource->engines.end() )
// 		kdDebug() << "engineio::Start: Can not find engine resource ID " << ID << endl;
	engineResource engineR = Resource->currentEngine;
	/* ...protocol */
	Protocol = engineR.Protocol;
	/* ...engine name */
	strcpy( FEATURE_MyName, engineR.Name.latin1() );
	(*engine) << engineR.Filename;
        
          
                    
/*
              nolevels = false;  */                         
        if (Resource->difficulty=="very_easy") {
          difficultyArgs = engineR.very_easy_args;
          
          if (difficultyArgs.isEmpty())
            nolevels = true;
         else
            nolevels = false;
          
        }      
        
        else if (Resource->difficulty=="easy")
          difficultyArgs = engineR.easy_args;
        else if (Resource->difficulty=="moderate")
          difficultyArgs = engineR.moderate_args;
        else if (Resource->difficulty=="hard")
          difficultyArgs = engineR.hard_args;
        else if (Resource->difficulty=="very_hard")
          difficultyArgs = engineR.very_hard_args;
        kdDebug() << "engine arguments: " << engineR.Arguments +" " + difficultyArgs << endl;
        
	/* ...command line arguments */
	args = QStringList::split( QString(" "), engineR.Arguments + " " + difficultyArgs, FALSE );
	for( tmp = 0; tmp < args.count(); tmp++ )
		(*engine) << args[tmp];
	/* ...log file */
	if( !engineR.LogFile.isEmpty() )
	{
		Log = new QFile( engineR.LogFile );
		if( !Log->open( IO_WriteOnly | IO_Append ) )
			if( !Log->open( IO_WriteOnly ) )
				kdDebug() << "engineio::Start: Can not open " << engineR.LogFile << " for writing." << endl;
	}

	/* Showtime */
	if( !engine->start( KProcess::NotifyOnExit, KProcess::All ) )
	{
		kdDebug() << "engineio::Start: Can not run the engine: " << engineR.Filename << endl;
		return;
	}
	InitTimer = 21;
	if( ID == ENGINE_WHITE ) Army = WHITE;
	if( ID == ENGINE_WHITE_BK ) Army = WHITE;
	if( ID == ENGINE_BLACK ) Army = BLACK;
	if( ID == ENGINE_BLACK_BK ) Army = BLACK;
	connect( engine, SIGNAL( wroteStdin( KProcess* ) ), this, SLOT( SendClear( KProcess* ) ) );
	connect( engine, SIGNAL( receivedStdout( KProcess*, char*, int ) ), this, SLOT( Recv( KProcess*, char*, int ) ) );
	connect( engine, SIGNAL( receivedStderr( KProcess*, char*, int ) ), this, SLOT( Recv( KProcess*, char*, int ) ) );
	return;
}
///////////////////////////////////////
//
//	engineio::Kill
//
///////////////////////////////////////
void engineio::Kill( void )
{
	SendCMD( CMD_Exit );
	if( FEATURE_SIGTERM )
		engine->kill();
	if( engine != NULL ) delete engine;
}
///////////////////////////////////////
//
//	engineio::matchID
//
///////////////////////////////////////
int engineio::matchID(void)
{
	return MatchID;
}
///////////////////////////////////////
//
//	engineio::setMatchID
//
///////////////////////////////////////
void engineio::setMatchID( const int ID )
{
	MatchID = ID;
}
///////////////////////////////////////
//
//	engineio::Send
//
///////////////////////////////////////
void engineio::Send( void )
{
	if( !engine->isRunning() ) return;
	if( FIFO_Out.isEmpty() ) return;
	if( !SafeToSend ) return;

	/* Interrupt those engines that want or need it */
	if( ( SendSIGINT ) && ( FEATURE_SIGINT ) && ( !Forced ) )
	{
		engine->kill( SIGINT );
		SendSIGINT = FALSE;
	}
	/* Write it */
	SafeToSend = FALSE;
        //kdDebug() << "Writing to STdin: " << FIFO_Out << endl;
	if( engine->writeStdin( FIFO_Out.latin1(), FIFO_Out.length() ) )
		FIFO_Out = "";
	else
		kdDebug() << "engineio::Send: Could not write to " << FEATURE_MyName << endl;
}
///////////////////////////////////////
//
//	engineio::Recv
//
///////////////////////////////////////
void engineio::Recv( KProcess*, char *buffer, int buflen )
{
 	QString  lineStr;
         lineStr = buffer;
         lineStr = lineStr.left(buflen);
         FIFO_Tmp = FIFO_In + ' ' + lineStr;
         
 	FIFO_Tmp.replace( QRegExp( "\\n" ), " [KI:NEW] " );
 	FIFO_Tmp.replace( QRegExp( "\\r" ), " [KI:NEW] " );
         FIFO_In = FIFO_Tmp.stripWhiteSpace();
         kdDebug() << "Received: " << FIFO_Tmp << endl;
 	if( QString( buffer ).contains( QChar('\n') ) ) SendSIGINT = TRUE;
	return;
}
///////////////////////////////////////
//
//	engineio::ReadFIFO
//
///////////////////////////////////////
void engineio::ReadFIFO( void )
{
	kdDebug() << "engineio::ReadFIFO start" << endl;
	kdDebug() << "read FIFO:" << FIFO_In << endl;
        int Position;

	FIFO_Token = "";
        
	if( FIFO_In.isEmpty() ) return;
        if( !FIFO_In.contains( QString("[KI:NEW]") ) ) return;
	//if (FIFO_In == "[KI:NEW]" || FIFO_In == " [KI:NEW] ") return;
       	
        kdDebug() << "read FIFO:" << FIFO_In << endl;
	Position = FIFO_In.find( ' ' );
        kdDebug() << "Position: " << Position << endl;
	/* If this is the last token */
	if( Position == -1 )
	{
                kdDebug() << "Last token" << endl;
		FIFO_Token = FIFO_In;
		FIFO_In = "";
		return;
	}
	FIFO_Tmp = FIFO_In;
	FIFO_Token = FIFO_Tmp.left( Position );
	FIFO_In = FIFO_Tmp.right( FIFO_Tmp.length() - Position - 1 );
	if( Log )
	{
		Log->writeBlock( FIFO_Token.latin1(), FIFO_Token.length() );
		Log->putch( '\n' );
	}
        kdDebug() << "engineio::ReadFIFO end" << endl;
	return;
}
///////////////////////////////////////
//
//	engineio::WriteFIFO
//
///////////////////////////////////////
void engineio::WriteFIFO( void )
{
	//kdDebug() << "engineio::WriteFIFO start" << endl;
	if( FIFO_Token.isEmpty() ) return;
	if( FIFO_Token.right(1) != "\n" ) FIFO_Token += "\n";
	FIFO_Out += FIFO_Token;
	Send();
	/* Print the output to the log file */
	if( Log )
	{
		FIFO_Tmp = "<< " + FIFO_Token;
		Log->writeBlock( FIFO_Tmp.latin1(), FIFO_Tmp.length() );
	}
       // kdDebug() << "engineio::WriteFIFO end" << endl;
	return;
}
void engineio::WriteFIFO( const char *Data )
{
	FIFO_Token = Data;
	WriteFIFO();
}
void engineio::WriteFIFO( const QString Data )
{
	FIFO_Token = Data;
	WriteFIFO();
}

///////////////////////////////////////
//
//	engineio::SendCMD
//
///////////////////////////////////////
void engineio::SendCMD( const int CMD, const int Value )
{
	kdDebug() << "engineio::SendCMD (const int, const int Value)" << endl;
        /* NOTE: Default cases on Protocol switches ALWAYS
			refers to Knights1 */
 	switch( CMD )
 	{
		/* Command: Init */
 		case CMD_Init:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "xboard" );
					WriteFIFO( "protover 3" );
					Poll();
					break;
				default:
					WriteFIFO( "knights 1" );
					break;
			}
 			break;
		/* Command: New Game */
 		case CMD_NewGame:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "new" );
					WriteFIFO( "random" );
					break;
				default:
					WriteFIFO( "newgame" );
					break;
			}
 			break;
		/* Command: Exit */
 		case CMD_Exit:
			switch( Protocol )
			{
				case XBoard:	// Fall Through
				default:
					WriteFIFO( "quit" );
					break;
			}
 			break;
		/* Command: Move Now */
 		case CMD_MoveNow:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "?" );
					break;
				default:
					WriteFIFO( "movenow" );
					break;
			}
 			break;
		/* Command: Move */
 		case CMD_Move:
			AcceptIllegal = TRUE;
			switch( Protocol )
			{
				case XBoard:
					FIFO_Token = "";
					if( FEATURE_UserMove ) FIFO_Token += "usermove ";
					if( FEATURE_SAN ) FIFO_Token += Move.SAN;
					else FIFO_Token += Move.CAN;
					kdDebug() << Move.SAN << " " << Move.CAN << endl;
					WriteFIFO();
					if( DelayedGo )
					{
						DelayedGo = FALSE;
						WriteFIFO( "go" );
						Forced = FALSE;
					}
					break;
				default:
					WriteFIFO( "mymove" );
					WriteFIFO( Move.CAN );
					break;
			}
 			break;
		/* Command: Pause */
		case CMD_Pause:
			switch( Protocol )
			{
				case XBoard:
					if( FEATURE_Pause ) WriteFIFO( "pause" );
					else
					{
						WriteFIFO( "force" );
						Forced = TRUE;
					}
					break;
				default:
					WriteFIFO( "pause" );
					break;
			}
			break;
		/* Command: Resume */
		case CMD_Resume:
			switch( Protocol )
			{
				case XBoard:
					if( FEATURE_Pause ) WriteFIFO( "resume" );
					else
					{
						if( Turn == Army )
						{
							WriteFIFO( "go" );
							Forced = FALSE;
						}
						else DelayedGo = TRUE;
					}
					break;
				default:
					WriteFIFO( "resume" );
					break;
			}
			break;
		/* Command: Play White */
 		case CMD_Play_White:
			Army = WHITE;
			switch( Protocol )
			{
				case XBoard:
					if( Turn == BLACK )
					{
						if( FEATURE_PlayOther ) { WriteFIFO( "playother" ); break; }
						if( FEATURE_Colors ) { WriteFIFO( "black" ); break; }
						WriteFIFO( "force" );
						Forced = TRUE;
						DelayedGo = TRUE;
						break;
					}
					if( Turn == WHITE )
					{
						WriteFIFO( "force" );
						if( FEATURE_Colors ) WriteFIFO( "white" );
						WriteFIFO( "go" );
						break;
					}
					break;
				default:
					WriteFIFO( "play white" );
					break;
			}
 			break;
		/* Command: Play Black */
 		case CMD_Play_Black:
			Army = BLACK;
			switch( Protocol )
			{
				case XBoard:
					if( Turn == WHITE )
					{
						if( FEATURE_PlayOther ) { WriteFIFO( "playother" ); break; }
						if( FEATURE_Colors ) { WriteFIFO( "white" ); break; }
						WriteFIFO( "force" );
						Forced = TRUE;
						DelayedGo = TRUE;
						break;
					}
					if( Turn == BLACK )
					{
						WriteFIFO( "force" );
						if( FEATURE_Colors ) WriteFIFO( "black" );
						WriteFIFO( "go" );
						break;
					}
					break;
				default:
					WriteFIFO( "play black" );
					break;
			}
 			break;
		/* Command: Result White */
 		case CMD_Result_White:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "result 1-0 {White Mates}" );
					break;
				default:
					WriteFIFO( "winner white" );
					break;
			}
// 			if( Army == WHITE ) engineR.Wins++;
// 			else engineR.Losses++;
 			break;
		/* Command: Result Black */
 		case CMD_Result_Black:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "result 0-1 {Black Mates}" );
					break;
				default:
					WriteFIFO( "winner black" );
					break;
			}
// 			if( Army == BLACK ) engineR.Wins++;
// 			else engineR.Losses++;
 			break;
		/* Command: Result Draw */
 		case CMD_Result_Draw:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "result 1/2-1/2 {Draw Game}" );
					break;
				default:
					WriteFIFO( "winner draw" );
					break;
			}
			//engineR.Draws++;
 			break;
		/* Command: Your Time */
		case CMD_Your_Time:
			switch( Protocol )
			{
				case XBoard:
					if( FEATURE_Time ) WriteFIFO( QString( "time %1" ).arg( Value ) );
					break;
				default:
					WriteFIFO( "yourclock" );
					WriteFIFO( QString("%1").arg( Value ) );
					break;
			}
		break;
		/* Command: Enemy Time */
		case CMD_Enemy_Time:
			switch( Protocol )
			{
				case XBoard:
					if( FEATURE_Time ) WriteFIFO( QString( "otim %1" ).arg( Value ) );
					break;
				default:
					WriteFIFO( "otherclock" );
					WriteFIFO( QString("%1").arg( Value ) );
					break;
			}
		break;
		/* Command: Offer Draw */
		case CMD_Offer_Draw:
			switch( Protocol )
			{
				case XBoard:
					if( FEATURE_Draw ) WriteFIFO( "draw" );
					break;
				default:
					WriteFIFO( "draw" );
					break;
			}
		break;
		/* Command: Book Mode */
		case CMD_Book_Mode:
			switch( Protocol )
			{
				case XBoard:
					if( FEATURE_BookEngine == 0 ) break;
					WriteFIFO( "bookengine" );
					break;
				default:
					WriteFIFO( "bookengine" );
					break;
			}
		break;
		/* Command: Check Book */
		case CMD_Check_Book:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "go" );
					Forced = FALSE;
					break;
				default:
					WriteFIFO( "go" );
					break;
			}
		break;
		/* Command: Ponder */
		case CMD_Ponder:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "easy" );
					WriteFIFO( "hard" );
					break;
				default:
					WriteFIFO( "pondering on" );
					break;
			}
		break;
		/* Command: No Pondering */
		case CMD_No_Pondering:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "hard" );
					WriteFIFO( "easy" );
					break;
				default:
					WriteFIFO( "pondering off" );
					break;
			}
		break;
		/* Command: Retract Move */
		case CMD_Retract_Move:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "remove" );
					break;
				default:
					WriteFIFO( "retract" );
					break;
			}
		break;
		/* Command: Hint */
		case CMD_Hint:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "hint" );
					break;
				default:
					WriteFIFO( "hint" );
					break;
			}
		break;
		/* Command: Listen */
		case CMD_Listen:
			switch( Protocol )
			{
				case XBoard:
					WriteFIFO( "force" );
					Forced = TRUE;
					DelayedGo = FALSE;
					break;
				default:
					WriteFIFO( "listen" );
					break;
			}
		break;
		/* Command: Play */
		case CMD_Play:
			switch( Protocol )
			{
				case XBoard:
					if( Army == Turn )
					{
						WriteFIFO( "go" );
						Forced = FALSE;
					}
					else
						DelayedGo = TRUE;
					break;
		break;
				default:
					WriteFIFO( "play" );
					break;
			}
		break;
		/* Command: Resign */
		case CMD_Resign:
			if( Army == WHITE ) SendCMD( CMD_Result_White );
			if( Army == BLACK ) SendCMD( CMD_Result_Black );
			break;
                                                              
 		default:
 			break;
 	}
/*	Cut 'n' Paste =)
		case CMD_:
			switch( Protocol )
			{
				case XBoard:
					break;
				default:
					break;
			}
		break;
*/
}
void engineio::SendCMD( const int CMD, const QString Value )
{

  kdDebug() << "engineio::SendCMD (const int, const QString)" << endl;
	/* NOTE: Default cases on Protocol switches ALWAYS
			refers to Knights1 */
 	switch( CMD )
 	{    
            case CMD_WriteText:
               switch(Protocol)
               {
               	case XBoard:
                	{
                        WriteFIFO(Value);
                        break;
                        }

                        
              case CMD_Set_Board:
                        kdDebug() << "engineio::SendCMD (const int, const QString)" << endl;
                        WriteFIFO(QString("setboard %1").arg(Value));
                        break;                        
                                       
               
               }
 		default:
 			break;
	}
}
///////////////////////////////////////
//
//	engineio::Poll
//
///////////////////////////////////////
int engineio::Poll(void)
{
 	//kdDebug() << "engineio::Poll" << endl;
	
	if( InitTimer ) InitTimer--;
	Send();

	ReadFIFO();
 	while( !FIFO_Token.isEmpty() )
 	{
		switch( Protocol )
		{
			/*				*/
			/* XBOARD */
			/*				*/
			case XBoard:
				if( ( FIFO_Token == "Illegal" ) || ( FIFO_Token == "illegal" )  )
				{
                                	
					kdDebug() << "FIFO_Token: " << FIFO_Token <<  endl;
                                        while( FIFO_Token != "[KI:NEW]" ) ReadFIFO();
					if( AcceptIllegal )
					{
						AcceptIllegal = FALSE;
                                                
						return CMD_Illegal;
					}
				}
				/* A Move */
		 		if( FIFO_Token == "move" )
		 		{
 					ReadFIFO();
					ClearMove();
					strcpy( Move.SAN, FIFO_Token.latin1() );
					strcpy( Move.CAN, FIFO_Token.latin1() );
					if( Team )
					{
						WriteFIFO( "force" );
						Forced = TRUE;
					}
					return CMD_Move;
				}
				/* A Move ( Old Variation ) */
				if( FIFO_Token.contains( QRegExp("^\\d+\\.$") ) )
				{
					ReadFIFO();
					if( FIFO_Token == "..." )	// Computer's Move
					{
 						ReadFIFO();
						ClearMove();
						strcpy( Move.SAN, FIFO_Token.latin1() );
						strcpy( Move.CAN, FIFO_Token.latin1() );
						if( Team )
						{
							WriteFIFO( "force" );
							Forced = TRUE;
						}
						return CMD_Move;
					}
				}
				/* Hint */
		 		if( FIFO_Token == "Hint:" )
		 		{
					ReadFIFO();
					ClearMove();
					strcpy( Move.SAN, FIFO_Token.latin1() );
					return CMD_Hint;
				}
				/* Offer Draw */
		 		if( FIFO_Token == "offer" )
		 		{
					ReadFIFO();
					if( FIFO_Token == "draw" ) return CMD_Offer_Draw;
				}
				/* Out of Book */
		 		if( FIFO_Token == "outofbook" )
		 		{
					WriteFIFO( "force" );
					Forced = TRUE;
					return CMD_Out_Of_Book;
				}
				/* Tell User */
		 		if( FIFO_Token == "telluser" )
		 		{
					Inbound = "";
					ReadFIFO();
					while( FIFO_Token != "[KI:NEW]" )
					{
						Inbound += QString( FIFO_Token + " " );
						ReadFIFO();
					}
					return CMD_Tell_User;
				}
				/* Tell User Error */
		 		if( FIFO_Token == "tellusererror" )
		 		{
					Inbound = "";
					ReadFIFO();
					while( FIFO_Token != "[KI:NEW]" )
					{
						Inbound += QString( FIFO_Token + " " );
						ReadFIFO();
					}
					return CMD_Tell_User_Error;
				}
				/* Parse Features */
				if( !InitTimer ) break;
 				if( FIFO_Token == "done=1" )
				{
					InitTimer = 0;
					break;
				}
				/* This buys you 10 minutes */
 				if( FIFO_Token == "done=0" )
				{
					InitTimer += 6000;
					break;
				}
 				if( FIFO_Token == "feature" ) break;
 				if( FIFO_Token.left(3) == "ics" )
 				{
 					FEATURE_ICS = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted ics" );
 					break;
 				}
 				if( FIFO_Token.left(3) == "san" )
 				{
 					FEATURE_SAN = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted san" );
 					break;
 				}
 				if( FIFO_Token.left(4) == "draw" )
 				{
 					FEATURE_Draw = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted draw" );
 					break;
 				}
 				if( FIFO_Token.left(4) == "name" )
 				{
 					FEATURE_Name = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted name" );
 					break;
 				}
 				if( FIFO_Token.left(4) == "ping" )
 				{
 					FEATURE_Ping = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted ping" );
 					break;
 				}
 				if( FIFO_Token.left(4) == "time" )
 				{
 					FEATURE_Time = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted time" );
 					break;
 				}
 				if( FIFO_Token.left(5) == "level" )
 				{
 					FEATURE_Level = QString( FIFO_Token.right( FIFO_Token.length() - 6 ) ).toInt();
 					WriteFIFO( "accepted level" );
 					break;
 				}
 				if( FIFO_Token.left(5) == "pause" )
 				{
 					FEATURE_Pause = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted pause" );
 					break;
 				}
 				if( FIFO_Token.left(5) == "reuse" )
 				{
 					FEATURE_Reuse = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted reuse" );
 					break;
 				}
 				if( FIFO_Token.left(6) == "colors" )
 				{
 					FEATURE_Colors = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted colors" );
 					break;
 				}
 				if( FIFO_Token.left(6) == "myname" )
 				{
 					strcpy( FEATURE_MyName, QString( FIFO_Token.right( FIFO_Token.length() - 8 ) ) );
 					FEATURE_MyName[ strlen( FEATURE_MyName ) - 1 ] = 0;
 					WriteFIFO( "accepted myname" );
 					break;
 				}
 				if( FIFO_Token.left(6) == "sigint" )
 				{
 					FEATURE_SIGINT = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted sigint" );
 					break;
 				}
 				if( FIFO_Token.left(7) == "analyze" )
 				{
 					FEATURE_Analyze = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted analyze" );
 					break;
 				}
 				if( FIFO_Token.left(7) == "sigterm" )
 				{
 					FEATURE_SIGTERM = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted sigterm" );
 					break;
 				}
 				if( FIFO_Token.left(8) == "setboard" )
 				{
 					FEATURE_SetBoard = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted setboard" );
 					break;
 				}
 				if( FIFO_Token.left(8) == "usermove" )
 				{
 					FEATURE_UserMove = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted usermove" );
 					break;
 				}
 				if( FIFO_Token.left(8) == "variants" )
 				{
 					FIFO_Tmp = FIFO_Token.right( FIFO_Token.length() - 10 );
 					// This must be finished
 					WriteFIFO( "accepted variants" );
 					break;
 				}
 				if( FIFO_Token.left(9) == "playother" )
 				{
 					FEATURE_PlayOther = QString( FIFO_Token.right(1) ).toInt();
 					WriteFIFO( "accepted playother" );
 					break;
 				}
 				if( FIFO_Token.left(10) == "bookengine" )
 				{
 					FEATURE_BookEngine = QString( FIFO_Token.right( FIFO_Token.length() - 11 ) ).toInt();
 					WriteFIFO( "accepted bookengine" );
 					break;
 				}
				break;
			/*					*/
			/* KNIGHTS 1*/
			/*					*/
			case Knights1:	// Knights1 fall through
			default:
				break;
		}
		ReadFIFO();
 	}
        
	return CMD_None;
}

///////////////////////////////////////
//
//	engineio::ClearMove
//
///////////////////////////////////////
void engineio::ClearMove(void)
{
	Move.fromFile = Null;
	Move.fromRank = Null;
	Move.toFile = Null;
	Move.toRank = Null;
	Move.Promote = Null;
	Move.ManTaken = Null;
	Move.NAG = 0;
}
void engineio::SendClear( KProcess* )
{
	SafeToSend = TRUE;
	if( !FIFO_Out.isEmpty() )
		Send();
}

