/***************************************************************************
                          internetio.cpp  -  description
                             -------------------
    begin                : Thu Aug 16 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "internetio.h"
#include <qregexp.h>
#include <kmessagebox.h>
//#include "../config.h"

static const int TXT_Std = 0;
static const int TXT_Pvt = 1;
static const int TXT_Ch  = 2;
static const int TXT_Sht = 3;
static const int TXT_Wsp = 4;
static const int TXT_Not = 5;

internetio::internetio( QWidget *parent, const char *name, resource *Rsrc ) : QObject(parent,name)
{
	Resource = Rsrc;
	Challenge = NULL;
	Sock = NULL;
	Dns = NULL;
	Timeseal = NULL;
	LoginStage = 0;
	SeekTimer = 0;
	extendedData = 0;
	LastType = TXT_Std;
	Log = NULL;
	prevMovesMatch = Null;
	
	NetConComm = new netconcomm( parent, "netconcomm" );
	connect( NetConComm, SIGNAL( newMatch( const QString& ) ), this, SLOT(slot_newMatch( const QString& ) ) );
	L_Match.setAutoDelete( TRUE );
	
	/* Get and parse server config from resource */
// 	for( serversIT = Resource->servers.begin(); serversIT != Resource->servers.end(); ++serversIT )
// 		if( (*serversIT).CurrentRef ) break;
// 	if( serversI0T == Resource->servers.end() )
// 		kdDebug() << "serverio::serverio: Can not find server resource CurrentRef " << endl;
	/* If username or password is empty, open a login dialog */
        serversIT = Resource->servers.begin();
	if( (*serversIT).UserName.isEmpty() || (*serversIT).Password.isEmpty() )
	{
		LoginDlg = new dlg_login( 0, "LoginDialog", Resource, serversIT );
		connect( LoginDlg, SIGNAL( okClicked() ), this, SLOT( finishInit() ) );
	}
	else
		finishInit();
}
internetio::~internetio()
{
	if( Log )
	{
		Log->close();
		delete Log;
	}
	if( Timeseal != NULL )
	{
			Timeseal->kill();
		delete Timeseal;
	}
	if( Dns != NULL ) delete Dns;
	if( Sock != NULL ) delete Sock;
	delete NetConComm;
}
///////////////////////////////////////
//
//	internetio::finishInit
//
///////////////////////////////////////
void internetio::finishInit( void )
{
	/* This is a workaround for the change in KDE modality.
			In KDE2, a modal dialog would freeze the parent widget
			in it's tracks. In KDE3, a modal dialog just keeps you
			from interfacing with the widget: The code continues to run.
			This broke the relationship between internetio and dlg_login.
			This method fixes that. */

/* Get login data */
qDebug("internetio::finishInit");
	Login = (*serversIT).UserName;
	Password = (*serversIT).Password;
	Host = (*serversIT).URL;
	Port = (*serversIT).Port;
	TimesealFilename = (*serversIT).Timeseal;
	if( Port == 0 ) Port = 5000;
	/* ...log file */
	if( !(*serversIT).LogFile.isEmpty() )
	{
		Log = new QFile( (*serversIT).LogFile );
		if( !Log->open( IO_WriteOnly | IO_Append ) )
		{
			if( !Log->open( IO_WriteOnly ) ) kdDebug() << "Can not open " << (*serversIT).LogFile << " for writing." << endl;
		}
	}
	/* Get the IP Address from this hostname */
	Dns = new QDns( Host );
	connect( Dns, SIGNAL( resultsReady() ), this, SLOT( slot_getAddress() ) );
}
///////////////////////////////////////
//
//	internetio::findICSMatchID
//
///////////////////////////////////////
bool internetio::findICSMatchID( const int ID )
{
  unsigned int Index;
	
	for( Index = 0; Index < L_Match.count(); Index++ )
	{
		L_Match.at( Index );
		if( L_Match.current()->serverID == ID ) return TRUE;
	}
	kdDebug() << "internetio::findICSMatchID: Can not locate match with ICS game #" << ID << endl;
	return FALSE;
}
///////////////////////////////////////
//
//	internetio::findKnightsMatchID
//
///////////////////////////////////////
void internetio::findKnightsMatchID( const int ID )
{
  unsigned int Index;
	
	for( Index = 0; Index < L_Match.count(); Index++ )
	{
		L_Match.at( Index );
		if( L_Match.current()->knightsID == ID ) return;
	}
	L_Match.at( 0 );
}
///////////////////////////////////////
//
//	internetio::matchID
//
///////////////////////////////////////
int internetio::matchID( void )
{
	if( !L_Match.count() )
	{
		kdDebug() << "internetio::matchID: Tried to get match#, but no matches exsist." << endl;
		return Null;
	}
	return L_Match.current()->knightsID;
}
bool internetio::matchID( const int ID )
{
  int Index;
	
	for( Index = 0; Index < (signed)L_Match.count(); Index++ )
	{
		L_Match.at( Index );
		if( L_Match.current()->knightsID == ID ) return TRUE;
	}
	return FALSE;
}
///////////////////////////////////////
//
//	internetio::addMatchID
//
///////////////////////////////////////
void internetio::addMatchID( const int ID )
{
	MatchRecord *newMatch;

	newMatch = new MatchRecord;
	newMatch->knightsID = ID;
	newMatch->serverID = NewICSGameNumber;
	newMatch->observing = FALSE;
	newMatch->examining = FALSE;
	newMatch->finished = FALSE;
	L_Match.append( newMatch );
	NewICSGameNumber = Null;
}
///////////////////////////////////////
//
//	internetio::removeMatchID
//
///////////////////////////////////////
void internetio::removeMatchID( const int ID )
{
  int Index;
	
	for( Index = 0; Index < (signed)L_Match.count(); Index++ )
	{
		L_Match.at( Index );
		if( L_Match.current()->knightsID == ID )
		{
			/* Tell ICS that we're leaving */
			if( !L_Match.current()->finished )
			{
				if( L_Match.current()->observing )
					Send( QString("unobserve %1").arg( L_Match.current()->serverID ) );
				else
					if( L_Match.current()->examining )
						Send( QString("unexamine") );
/* NOTICE ME!!! CHANGE FOR THE NEXT RELEASE!!! SEE KNIGHTS-DEVEL */
//					else
//						Send( QString("resign") );
			}
			L_Match.remove();
			Index = 0;
		}
	}
}
///////////////////////////////////////
//
//	internetio::Send
//
///////////////////////////////////////
void internetio::Send( const QString &msg )
{
	QString tmp(msg);
	unsigned int len;
	
	if( Sock->state() != QSocket::Connection )
	{
		kdDebug() << "internetio::Send: Can not write to socket... Not connected." << endl;
		return;
	}
	/* Attach events to specific outgoing text */
	if( ( tmp == "accept" ) ||
			( tmp == "decline" ) ||
			( tmp.left(6) == "match " ) )
			if( Challenge != NULL )
			{
				delete Challenge;
				nullifyChallenge();
			}
	if( tmp.right(1) != "\n" ) tmp += "\n";
	len = Sock->writeBlock( tmp.latin1(), tmp.length() );
	if( Log )
	{
		Log->writeBlock( QString("<< ").latin1(), 3 );
		Log->writeBlock( tmp.latin1(), len );
	}
	if( len < tmp.length() ) kdDebug() << "internetio::Send: Failed to write full block of data to socket." << endl;
}
///////////////////////////////////////
//
//	internetio::slot_getAddress
//
///////////////////////////////////////
void internetio::slot_getAddress( void )
{
	QValueList<QHostAddress> L_Address;
	QString addressString;

	L_Address = Dns->addresses();
	if( !L_Address.count() )
	{
		KMessageBox::sorry( (QWidget*)parent(), i18n( "Knights can not find %1.\nPlease make sure the address was entered correctly and that you are connected to the internet." ).arg(Host), i18n( "Can not find server." ) );
		return;
	}
	addressString = L_Address[0].toString();
	/* Connect directly or use Timeseal? */
	if( !TimesealFilename.isEmpty() )
	{
		Timeseal = new QProcess();
		Timeseal->addArgument(TimesealFilename);
                Timeseal->addArgument(addressString);
                Timeseal->addArgument(QString().setNum( Port ));
                Timeseal->addArgument("-p");
                Timeseal->addArgument(QString().setNum( Port + 1 ));
                 //<< addressString << QString().setNum( Port ) << QString( "-p" ) << QString().setNum( Port + 1 );
		if( !Timeseal->start() )
		{
			KMessageBox::sorry( (QWidget*)parent(), i18n( "Knights can not start Timeseal.\nPlease make sure you have the correct path and filename configured." ), i18n( "Can not find Timeseal." ) );
			return;
		}
		/* Begin the connection */
		Sock = new QSocket( this );
		Sock->flush();
		Sock->connectToHost( QString("localhost"), Port + 1 );
	}
	else
	{
		/* Begin the connection */
		Sock = new QSocket( this );
		Sock->flush();
		Sock->connectToHost( Host, Port );
	}
	disconnect( Dns, SIGNAL( resultsReady() ), this, SLOT( slot_getAddress() ) );
}
///////////////////////////////////////
//
//	internetio::Poll
//
///////////////////////////////////////
int internetio::Poll( void )
{	
	QStringList fields;
	QString tmp;
	if( Sock == NULL ) return CMD_None;
	if( Sock->state() != QSocket::Connection ) return CMD_None;
	/* update Seek */
 	if( NetConComm->seeking() )
 	{
 		SeekTimer++;
 		if( SeekTimer > Resource->Seek_Timer )
 		{
 			SeekTimer = 0;
 			Send( QString("sought") );
 		}
 	}
 	else
 		SeekTimer = Resource->Seek_Timer - 1;
	/* Prime reading loop */
	while( readLine() && ( LoginStage > 2 ) )
	{
  	/* Check to see if we're getting a game board */
  	if( Inbound.left(9) == QString("&#60;12> ") )
  	{
  		if( ParseStyle12() == TRUE ) return CMD_Move;
			return CMD_None;
  	}
		/* Check to see if a game has been forfeit */
		/* Check to see if someone resigned */
		extendedData = 0;
		if( Inbound.contains( QRegExp("^\\{?Game \\d+") ) )
		{
			if( Inbound.contains( " forfeits by disconnection} ", TRUE ) )
				extendedData = 1;
			if( Inbound.contains( " forfeits on time} ", TRUE ) )
				extendedData = 2;
			if( Inbound.contains( " resigns} ", TRUE ) )
				extendedData = 3;
			if( Inbound.contains( " Game drawn by mutual agreement} ", TRUE ) )
				extendedData = 4;
			if( Inbound.contains( ", has lost contact or quit.", TRUE ) )
				extendedData = 5;
			if( Inbound.contains( " checkmated} ", TRUE ) )
				extendedData = 6;
		}
		if( extendedData != 0 )
 		{
 			fields = QStringList::split( QChar(' '), Inbound, FALSE );
 			if( !findICSMatchID( fields[1].toInt() ) )
 				return CMD_None;
 			tmp = fields[ fields.count() - 1 ].stripWhiteSpace();
 			if( tmp == "1-0" )
 				return CMD_Result_White;
 			if( tmp == "0-1" )
 				return CMD_Result_Black;
 			if( tmp == "1/2-1/2" )
 				return CMD_Result_Draw;
 		}
		if( Inbound.contains( QRegExp( "^Illegal move \\(.+\\)\\.$" ) ) )
			return CMD_Illegal;
		/* Parse the Input */
  	if( prevMovesMatch == Null )
			ParseData();
		else
		{
			ParseMovelist();
			return CMD_Read_Movelist;
		}
  	if( Inbound.isEmpty() ) continue;
		NetConComm->setMessage( Inbound );
  	return CMD_Raw_Input;
	}
 	/* Handle Login */
 	if( LoginStage != 3 )
 	{
 		switch( LoginStage )
 		{
 			case 0:
 				Send( Login );
 				break;
 			case 1:
 				Send( Password );
 				break;
 			case 2:
				sendCMD( CMD_Reset_Server );
 				break;
 			default:
 				break;
 		}
 		LoginStage++;
 	}
 	return CMD_None;
}
///////////////////////////////////////
//
//	internetio::readLine
//
///////////////////////////////////////
bool internetio::readLine( void )
{
	if( Sock->canReadLine() != TRUE ) return FALSE;
	Inbound = Sock->readLine();
	if( Log )
	{
		Log->writeBlock( Inbound.latin1(), Inbound.length() );
	}
	/* Remove Line Breaks & Bells */
	Inbound.replace( QRegExp("\\n"), "" );
	Inbound.replace( QRegExp("\\r"), "" );
	Inbound.replace( QRegExp("\\x0007") , "" );
	/* Replace misc characters with rich-text friendly counterparts */
	Inbound.replace( QRegExp("\\x003c"), "&#60;" );
	Inbound.replace( QRegExp("\\x007c"), "&#124;" );
	return TRUE;
}
///////////////////////////////////////
//
//	internetio::sendCMD
//
///////////////////////////////////////
void internetio::sendCMD( const int CMD, const int KnightsMatch )
{
	if( KnightsMatch != -1 ) findKnightsMatchID( KnightsMatch );
 	switch( CMD )
 	{
		/* Command: Offer Draw */
		case CMD_Offer_Draw:
			Send( "draw" );
			break;
		/* Command: Reject Draw */
		case CMD_Reject_Draw:
			Send( "decline" );
			break;
		/* Command: Resign */
		case CMD_Resign:
			Send( "resign" );
			break;
		/* Command: Call Flag */
		case CMD_Call_Flag:
			Send( "flag" );
			break;
		/* Command: Examine Forward */
		case CMD_Examine_Forward:
			Send( "forward 1" );
			break;
		/* Command: Examine Backward */
		case CMD_Examine_Backward:
			Send( "backward 1" );
			break;
		/* Catch all Finished Matches */
		case CMD_Result_Draw:
		case CMD_Result_White:
		case CMD_Result_Black:
			L_Match.current()->finished = TRUE;
			break;
		/* Reset Server */
		case CMD_Reset_Server:
			Send( "set style 12" );
			Send( QString("set interface Knights %1").arg("0.6.4") );
			Send( QString("set private %1").arg( Resource->OPTION_Private ) );
			Send( QString("set kibitz %1").arg( Resource->OPTION_Kibitz ) );
			Send( QString("set tell %1").arg( Resource->OPTION_Tell ) );
			Send( QString("set shout %1").arg( Resource->OPTION_Shout ) );
			Send( QString("set seek %1").arg( Resource->OPTION_Seek ) );
			Send( QString("set tolerance %1").arg( Resource->OPTION_Profanity ) );
			break;
 		default:
 			break;
	}
/*	Cut 'n' Paste =)
		case CMD_:
		break;
*/
}
///////////////////////////////////////
//
//	internetio::ParseData
//
///////////////////////////////////////
void internetio::ParseData( void )
{
	int Index, Index2;
	QString tmp;

	/* Remove prompt */
	Inbound.replace( QRegExp( "a?d?f?g?s?z?ics% " ), "" );
	Inbound.replace( QRegExp( "cex% " ), "" );
	Inbound.replace( QRegExp( "chess% " ), "" );
	/* Look for bad logins */
	if( ( Inbound == "Sorry, names can only consist of lower and upper case letters.  Try again." ) ||
			( Inbound == "**** Invalid password! ****" ) )
			{
				Sock->close();
				emit shutdown();
				Inbound = "";
				return;
			}
	/* Continuation of last line? */
	if( Inbound.left(4) == "\\   " )
	{
		switch( LastType )
		{
			case TXT_Pvt:
				Inbound.prepend( "<K_PVT>" );
				Inbound.append( "</K_PVT>" );
				break;
			case TXT_Wsp:
				Inbound.prepend( "<K_WSP>" );
				Inbound.append( "</K_WSP>" );
				break;
			case TXT_Sht:
				Inbound.prepend( "<K_SHT>" );
				Inbound.append( "</K_SHT>" );
				break;
			case TXT_Not:
				Inbound.prepend( "<K_NOT>" );
				Inbound.append( "</K_NOT>" );
				break;
			case TXT_Ch:
				Inbound.prepend( "<K_CH>" );
				Inbound.append( "</K_CH>" );
				break;
			case TXT_Std:	// Fall Through
			default:
				Inbound.prepend( "<K_STD>" );
				Inbound.append( "</K_STD>" );
				break;
		}
		return;
	}
	/* CHALLENGE */
	if( Inbound.left(11) == "Challenge: " )
	{
		Resource->play( SND_CHALLENGE );
		if( Challenge != NULL ) delete Challenge;
		Challenge = new dlg_challenge( 0, "Challenge", Resource );
		connect( Challenge, SIGNAL( destroyed() ), this, SLOT( nullifyChallenge() ) );
		connect( Challenge, SIGNAL( user1Clicked() ), this, SLOT( acceptChallenge() ) );
		connect( Challenge, SIGNAL( user2Clicked() ), this, SLOT( declineChallenge() ) );
		Challenge->setValues( Inbound, Login );
		Inbound = "";
		return;
	}
	if( ( Inbound.left(15) == "Challenge from " ) && ( Inbound.right(9) == " removed." ) )
		if( Challenge != NULL ) delete Challenge;
	/* "ADS DISPLAYED" MESSAGE */
	if( Inbound.contains( QRegExp("^\\d+\\s+ads? displayed.") ) )
	{
		NetConComm->updateSoughtList();
		Inbound = "";
		return;
	}
	/* SOUGHT GAME */
	if( Inbound.contains( QRegExp("^\\s*\\d+\\s+[\\+\\-0-9]+\\s+\\S+\\s+\\d+\\s+\\d+\\s+\\S+\\s+\\S+\\s+\\S+") ) )
	{
		NetConComm->addSoughtItem( Inbound );
		Inbound = "";
		return;
	}
	/* Begin a moves list */
	if( Inbound.left(18) == "Movelist for game " )
	{
		QStringList fields = QStringList::split( QChar(' '), Inbound, FALSE );
		Inbound = "";
		prevMovesMatch = fields[3].left( fields[3].length() - 1 ).toInt();
		return;
	}
	/* PRIVATE TELL */
	if( Inbound.contains( QRegExp(".+ tells you: .+") ) )
	{
		/* First grab the user name so we can auto-respond later */
		Index = Inbound.find( QString("(") );
		NetConComm->setPrivateSrc( Inbound.left( Index ) );
		
		Inbound.prepend( "<K_PVT>" );
		Inbound.append( "</K_PVT>" );
		LastType = TXT_Pvt;
		Resource->play( SND_TELL );
		return;
	}
	/* SAY */
	if( Inbound.contains( QRegExp(".+ says: .+") ) )
	{
		Inbound.prepend( "<K_PVT>" );
		Inbound.append( "</K_PVT>" );
		LastType = TXT_Pvt;
		Resource->play( SND_SAY );
		return;
	}
	/* WHISPER & KIBITZ */
	if( Inbound.contains( QRegExp(".+ whispers: .+") ) || Inbound.contains( QRegExp(".+ kibitzes: .+") ) )
	{
		Inbound.prepend( "<K_WSP>" );
		Inbound.append( "</K_WSP>" );
		LastType = TXT_Wsp;
		return;
	}
	/* Important System Messages: Use Whisper Color */
	if( Inbound.contains( QRegExp("declines the draw request\\.$" ) ) )
	{
		Inbound.prepend( "<K_WSP>" ); // Declined Draw Request
		Inbound.append( "</K_WSP>" );
		LastType = TXT_Wsp;
		return;
	}
	/* DRAW OFFER */
	if( Inbound.right(19) == " offers you a draw." )
	{
		Inbound.prepend( "<K_WSP>" ); // Offered a Draw
		Inbound.append( "</K_WSP>" );
		LastType = TXT_Wsp;
		Resource->play( SND_DRAW_OFFER );
	}
	if( Inbound.contains( QRegExp(".+rating adjustment:.+" ) ) )
	{
		Inbound.prepend( "<K_WSP>" ); // Ratings Adjustment
		Inbound.append( "</K_WSP>" );
		LastType = TXT_Wsp;
		return;
	}
	/* SHOUTS */
	if( Inbound.contains( QRegExp(" c?t?-?shouts: ") ) )
	{
		Inbound.prepend( "<K_SHT>" );
		Inbound.append( "</K_SHT>" );
		LastType = TXT_Sht;
		return;
	}
	/* NOTIFY */
	if( Inbound.contains( "Notification:" ) )
	{
		Inbound.prepend( "<K_NOT>" );
		Inbound.append( "</K_NOT>" );
		LastType = TXT_Not;
		Resource->play( SND_NOTIFICATION );
		return;
	}
	/* CHANNEL TELLS */
	if( Inbound.contains( QRegExp( ".\\(\\d+\\):" ) ) )
	{
		/* First grab the channel # so we can auto-respond later */
		Index2 = Inbound.find( QString("):") );
		Index = Inbound.findRev( QString("("), Index2 ) + 1;
		NetConComm->setChannelSrc( Inbound.mid( Index, Index2 - Index ) );
		
		Inbound.prepend( "<K_CH>" );
		Inbound.append( "</K_CH>" );
		LastType = TXT_Ch;
		return;
	}
	/* Standard Text */
	if( Inbound.contains( "login:" ) ) Inbound = "";
	if( Inbound.contains( "password:" ) ) Inbound = "";
	if( !Inbound.isEmpty() )
	{
		Inbound.prepend( "<K_STD>" );
		Inbound.append( "</K_STD>" );
		LastType = TXT_Std;
	}
}
///////////////////////////////////////
//
//	internetio::slot_newMatch
//
///////////////////////////////////////
void internetio::slot_newMatch( const QString& name )
{
	Send( QString("play %1").arg(name) );
	kdDebug() << name << endl;
}
///////////////////////////////////////
//
//	internetio::ParseStyle12
//
///////////////////////////////////////
bool internetio::ParseStyle12( void )
{
	bool newMatch( FALSE );
	QStringList fields;

	Pos.Board = "";
	fields = QStringList::split( QChar(' '), Inbound, FALSE );
 	Pos.Board = fields[1];	/* Internal Rank #7 */
 	Pos.Board += fields[2];	/* Internal Rank #6 */
 	Pos.Board += fields[3];	/* Internal Rank #5 */
 	Pos.Board += fields[4];	/* Internal Rank #4 */
 	Pos.Board += fields[5];	/* Internal Rank #3 */
 	Pos.Board += fields[6];	/* Internal Rank #2 */
 	Pos.Board += fields[7];	/* Internal Rank #1 */
 	Pos.Board += fields[8];	/* Internal Rank #0 */
	/* Color who's turn it is to move */
	if( fields[9] == "W" ) Pos.OnMove = WHITE;
	if( fields[9] == "B" ) Pos.OnMove = BLACK;
	/* Double Pawn Push indicator */
	Pos.PawnPushFile = fields[10].toShort();
	/* Can White Castle Short (boolean) */
	Pos.Board += fields[11];
	/* Can White Castle Long (boolean) */
	Pos.Board += fields[12];
	/* Can Black Castle Short (boolean) */
	Pos.Board += fields[13];
	/* Can Black Castle Long (boolean) */
	Pos.Board += fields[14];
	/* Number of moves since last irreversable ( 50-move rule ) */
	Pos.MoveCounter = fields[15].toInt();
	/* Game Number */
	if( findICSMatchID( fields[16].toInt() ) == FALSE )
	{
		/* This is a new game to us. Let's set it up */
		newMatch = TRUE;
		NewICSGameNumber = fields[16].toInt();
		switch( fields[19].toInt() )
		{
  		/* Examining a game */
  		case 2:
				emit newGame( BOTH );
  			break;
  		/* Observing a game */
  		case -2:
  		case 0:
				emit newGame( Null );
  			break;
  		/* Playing a game */
  		case -1:
				if( fields[9] == "W" )
					emit newGame( BLACK );
				else
					emit newGame( WHITE );
  			break;
  		/* Playing a game */
  		case 1:
				if( fields[9] == "B" )
					emit newGame( BLACK );
				else
					emit newGame( WHITE );
  			break;
  		default:
  			break;
		}
	}
	/* White's Name */
	Pos.WhiteName = fields[17];
	/* Black's Name */
	Pos.BlackName = fields[18];
	/* Our relation to this game */
	switch( fields[19].toInt() )
	{
		/* Examining a game */
		case 2:
			L_Match.current()->observing = FALSE;
			L_Match.current()->examining = TRUE;
			Pos.Mode = ICS_Examine;
			break;
		/* Observing a game */
		case -2:
		case 0:
			L_Match.current()->observing = TRUE;
			L_Match.current()->examining = FALSE;
			Pos.Mode = ICS_Observe;
			break;
		/* Playing a game */
		default:
			L_Match.current()->observing = FALSE;
			L_Match.current()->examining = FALSE;
			Pos.Mode = ICS_Normal;
			break;
	}
	/* BASE Time of the game */
//	fields[20];
	/* INCREMENT Time of the game */
//	fields[21];
	/* White's Material */
//	fields[22];
	/* Black's Material */
//	fields[23];
	/* White's Remaining Time */
	Pos.WhiteTime = fields[24].toInt() * 100;
	/* Black's Remaining Time */
	Pos.BlackTime = fields[25].toInt() * 100;
	/* Number of the move about to be made */
	if( ( fields[26].toInt() > 1 ) && newMatch )
	{
		/* Need to get move list */
		Send( QString("moves %1").arg( fields[16] ) );
		return FALSE;
	}
	/* Verbose Coordinate Notation of previous move ( USE AS CAN ) */
	Pos.CAN = fields[27].right( fields[27].length() - 2 );
	Pos.CAN.replace( QRegExp( "-" ), "" );
	/* Time taken to make previous move */
//	fields[28];
	/* SAN ( USE THIS ) */
	Pos.SAN = fields[29];
	/* Field Orientation */
//	fields[30];
	/* Clock Ticking */
	Pos.ClockTicking = fields[31].toInt();
	/* Lag Compensation */
	if( Pos.OnMove == WHITE ) Pos.WhiteTime += fields[32].toInt();
	else Pos.BlackTime += fields[32].toInt();
	/* Insert If/Then here to deal with a second line of Bughouse info */
	Inbound = "";
	return TRUE;
}
///////////////////////////////////////
//
//	internetio::ParsePlayer
//
///////////////////////////////////////
void internetio::ParsePlayer( QString Handle )
{
	player.Raw = Handle;
	/* SysAdmin */
	if( Handle.contains( QRegExp("\\(\\*\\)") ) )
	{
		player.SysAdmin = TRUE;
		Handle.replace( QRegExp("\\(\\*\\)"), QString("") );
	}
	else player.SysAdmin = FALSE;
	/* ServiceRep */
	if( Handle.contains( QRegExp("\\(SR\\)") ) )
	{
		player.ServiceRep = TRUE;
		Handle.replace( QRegExp("\\(SR\\)"), QString("") );
	}
	else player.ServiceRep = FALSE;
	/* Computer */
	if( Handle.contains( QRegExp("\\(C\\)") ) )
	{
		player.Computer = TRUE;
		Handle.replace( QRegExp("\\(C\\)"), QString("") );
	}
	else player.Computer = FALSE;
	/* Unregistered */
	if( Handle.contains( QRegExp("\\(U\\)") ) )
	{
		player.Unregistered = TRUE;
		Handle.replace( QRegExp("\\(U\\)"), QString("") );
	}
	else player.Unregistered = FALSE;
	return;
}
///////////////////////////////////////
//
//	internetio::nullifyChallenge
//
///////////////////////////////////////
void internetio::nullifyChallenge( void )
{
	Challenge = NULL;
}
///////////////////////////////////////
//
//	internetio::declineChallenge
//
///////////////////////////////////////
void internetio::declineChallenge( void )
{
	Send( "decline" );
	delete Challenge;
	Challenge = NULL;
}
///////////////////////////////////////
//
//	internetio::acceptChallenge
//
///////////////////////////////////////
void internetio::acceptChallenge( void )
{
	Send( Challenge->values() );
	delete Challenge;
	Challenge = NULL;
}
///////////////////////////////////////
//
//	internetio::ParseMovelist
//
///////////////////////////////////////
void internetio::ParseMovelist( void )
{
	QStringList fields = QStringList::split( QChar(' '), Inbound, FALSE );
	if( fields[0].left(1) == "{" )
		prevMovesMatch = Null;
	if( !fields[0].contains( QRegExp("\\d+\\.") ) )
		Inbound = "";
	return;
}

