/***************************************************************************
                          knightspixcache.cpp  -  description
                             -------------------
    begin                : Mon Aug 20 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "knightspixcache.h"

KnightsPixCache::KnightsPixCache()
{
	bytesTotal = 1024 * 1024;
	bytesUsed = 0;
}
KnightsPixCache::~KnightsPixCache()
{
	clear();
	bytesUsed = 0;
}
///////////////////////////////////////
//
//	KnightsPixCache::cacheLimit
//
///////////////////////////////////////
unsigned int KnightsPixCache::cacheLimit( void )
{
	return ( bytesTotal / 1024 );
}
///////////////////////////////////////
//
//	KnightsPixCache::setCacheLimit
//
///////////////////////////////////////
void KnightsPixCache::setCacheLimit( const unsigned int limit )
{
	bytesTotal = limit * 1024;
}
///////////////////////////////////////
//
//	KnightsPixCache::clear
//
///////////////////////////////////////
void KnightsPixCache::clear( void )
{
	list.clear();
}
///////////////////////////////////////
//
//	KnightsPixCache::clean
//
///////////////////////////////////////
void KnightsPixCache::cleanCache( const unsigned int space )
{
	pixmapList::Iterator	tmpIT;
	register unsigned int age;
	
	while( bytesUsed + space > bytesTotal )
	{
		if( list.count() == 0 ) return;
		age = 0;
		for( IT = list.begin(); IT != list.end(); ++IT )
		{
			if( (*IT).age > age )
			{
				age = (*IT).age;
				tmpIT = IT;
			}
		}
		bytesUsed -= (*tmpIT).bytes;
		list.remove( tmpIT );
	}
}
///////////////////////////////////////
//
//	KnightsPixCache::add
//
///////////////////////////////////////
void KnightsPixCache::add( const QString label, const QPixmap &pixmap )
{
	unsigned int tmp;
	cacheItem newItem;

	if( pixmap.isNull() ) return;
	tmp = ( pixmap.width() * pixmap.height() * pixmap.depth() ) / 8;
	if( ( bytesUsed + tmp ) > bytesTotal ) cleanCache(tmp);
	
	newItem.label = label;
	newItem.bytes = tmp;
	newItem.item = pixmap;
	newItem.age = 0;
	list.append( newItem );
}
///////////////////////////////////////
//
//	KnightsPixCache::find
//
///////////////////////////////////////
bool KnightsPixCache::find( const QString label, QPixmap &pixmap )
{
	bool status(FALSE);
	for( IT = list.begin(); IT != list.end(); ++IT )
	{
		if( (*IT).label == label )
		{
			pixmap = (*IT).item;
			(*IT).age = 0;
			status = TRUE;
		}
		else (*IT).age++;
	}
	return status;
}

