/***************************************************************************
                          match.cpp  -  description
                             -------------------
    begin                : Mon Jul 2 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qregexp.h>
//#include <kmessagebox.h>
#include "match.h"

///////////////////////////////////////
//
//	match::match
//
///////////////////////////////////////
match::match( QWidget *parent, const char *name, resource *Rsrc ) : QWidget(parent,name)
{
kdDebug() << "Match constructor 1" << endl;

	Resource = Rsrc;
	Record = new pgn( Resource );
	Logic = new logic( Resource );
        
/*	if( Resource->OPTION_3DBoard )
	{
		Board = NULL;
		 = new board3d( parent, "Board3d", Resource, Logic );
		Board3d->show();
		connect( Board3d, SIGNAL( LeftClick(int) ), this, SLOT( slot_Select(int) ) );
		connect( Board3d, SIGNAL( RightClick(int) ), this, SLOT( slot_Preview(int) ) );
	}
	else*/
      
	
	{
		Board = new board( parent, "Board", Resource, Logic );
		Board->show();
                kdDebug() << "Match constructor 2" << endl;
		connect( Board, SIGNAL( LeftClick(int) ), this, SLOT( slot_Select(int) ) );
                
		connect( Board, SIGNAL( RightClick(int) ), this, SLOT( slot_Preview(int) ) );
//	Board3d = NULL;
	}
        
	Clock = new chessclock( this, "Clock", Resource );
        
        Record->init();
        
	Logic->Init();
	
        Clock->Reset();
        
	clearSelections();
        
 kdDebug() << "Match constructor 3" << endl;       
	WhiteInput = PLAYERLOCAL;
	BlackInput = PLAYERLOCAL;
	Draw_Offered[WHITE] = FALSE;
	Draw_Offered[BLACK] = FALSE;
	Logic->OnMove = WHITE;
	Modified = FALSE;
	Loading = FALSE;
	Paused = FALSE;
	JustMoved = FALSE;
	preMoved = FALSE;
	Current = FALSE;
	Movelist = FALSE;
	loadTimer = 0;
	StatusCode = READY;
	StatusMessage = QString::null;
	connect( Record, SIGNAL( processMove(ChessMove) ), this, SLOT( slot_Move(ChessMove) ) );
	connect( Record, SIGNAL( processSpecial() ), this, SLOT( loadSpecial() ) );
	connect( Clock, SIGNAL( flagFell(const bool) ), this, SLOT( slot_flagFell(const bool) ) );
	connect( this, SIGNAL( setStatusBar( const int, const QString& ) ), this, SLOT( saveStatusBar( const int, const QString& ) ) );
        kdDebug() << "Match constructor end" << endl;
}
///////////////////////////////////////
//
//	match::~match
//
///////////////////////////////////////
match::~match()
{
	delete Clock;
	if( Board != NULL ) delete Board;
//	if( Board3d != NULL ) delete Board3d;
	delete Record;
	delete Logic;
}
///////////////////////////////////////
//
//	match::matchID
//
///////////////////////////////////////
int match::matchID(void)
{
	return MatchID;
}
///////////////////////////////////////
//
//	match::setMatchID
//
//////////////////////////////////////
void match::setMatchID( const int ID )
{
	MatchID = ID;
}
///////////////////////////////////////
//
//	match::setVisibility
//
///////////////////////////////////////
void match::setVisibility( const bool vis )
{
	if( vis == TRUE )
	{
		show();
/*		if( Resource->OPTION_3DBoard )
			Board3d->show();
		else*/
			Board->show();
		setEnabled( TRUE );
	}
	else
	{
		hide();
/*		if( Resource->OPTION_3DBoard )
			Board3d->hide();
		else*/
			Board->hide();
		setEnabled( FALSE );
	}
}
///////////////////////////////////////
//
//	match::setInput
//
///////////////////////////////////////
void match::setInput( const char White, const char Black )
{
	WhiteInput = White;
	BlackInput = Black;
	Logic->setInput( White, Black );
	switch( BlackInput )
	{
		case PLAYERLOCAL:
			setName( BLACK, Resource->Local_Player );
/*			if( Resource->OPTION_3DBoard )
				Board3d->setLocalArmy( BLACK );
			else*/
				Board->setLocalArmy( BLACK );
			Record->TAG_BlackType = "human";
			break;
		case PLAYERPC:
			Record->TAG_BlackType = "program";
			break;
		case PLAYERTCP:
			Record->TAG_Mode = "ICS";
			break;
		case PLAYEREMAIL:
			Record->TAG_Mode = "EM";
			break;
		default:
			Record->TAG_BlackType = "unknown";
			break;
	}
	switch( WhiteInput )
	{
		case PLAYERLOCAL:
			setName( WHITE, Resource->Local_Player );
			if( Resource->OPTION_3DBoard )
/*				Board3d->setLocalArmy( WHITE );
			else
				Board->setLocalArmy( WHITE );*/
			Record->TAG_BlackType = "human";
			break;
		case PLAYERPC:
			Record->TAG_BlackType = "program";
			break;
		case PLAYERTCP:
			Record->TAG_Mode = "ICS";
			break;
		case PLAYEREMAIL:
			Record->TAG_Mode = "EM";
			break;
		default:
			Record->TAG_BlackType = "unknown";
			break;
	}
}
///////////////////////////////////////
//
//	match::tick
//
///////////////////////////////////////
void match::tick(void)
{
	Clock->Tick();
	emit setClocks();
	if( Loading )
		if( loadTimer ) loadTimer--;
		else
			if( !Record->loadNext() )
			{
				/* All this is for a match we just finished loading */
				Loading = FALSE;
				
				if( Record->TAG_Result == "*" )
				{
					/* Unfinished Game */
					WhiteInput = Record->whitePlayer;
					BlackInput = Record->blackPlayer;
					Clock->Set( ( Record->whiteTime * 100 ), ( Record->blackTime * 100 ), onMove() );			
				}
				else
				{
					/* Finished Match */
					Clock->Pause();
					if( Record->TAG_Result == "1/2-1/2" )
						emit setStatusBar( GAME_DRAW );
					if( Record->TAG_Result == "1-0" )
						emit setStatusBar( WHITE_WIN );
					if( Record->TAG_Result == "0-1" )
						emit setStatusBar( BLACK_WIN );
				}
			}
}
///////////////////////////////////////
//
//	match::clearSelections
//
///////////////////////////////////////
void match::clearSelections( const char fromPtr )
{

	char tmp;

	for( tmp = 0; tmp < 64; tmp++ )
	{
			Logic->current[tmp].Note = NOTE_NONE;
/*			if( Resource->OPTION_3DBoard )
				Board3d->drawPosition( tmp );
			else*/
                        
				Board->drawPosition( tmp );
                                
	}
/*	if( Resource->OPTION_3DBoard )
		Board3d->commit();
	else*/
		Board->commit();
}
///////////////////////////////////////
//
//	match::clearMove
//
///////////////////////////////////////
void match::clearMove( const bool pre )
{
	if( pre )
	{
		preMove.fromFile = Null;
		preMove.fromRank = Null;
		preMove.toFile = Null;
		preMove.toRank = Null;
		preMove.Promote = Null;
		preMove.ManTaken = Null;
		preMove.NAG = 0;
	}
	else
	{
		chessMove.fromFile = Null;
		chessMove.fromRank = Null;
		chessMove.toFile = Null;
		chessMove.toRank = Null;
		chessMove.Promote = Null;
		chessMove.ManTaken = Null;
		chessMove.NAG = 0;
	}
}
///////////////////////////////////////
//
//	match::flip
//
///////////////////////////////////////
void match::flip(void)
{
/*	if( Resource->OPTION_3DBoard )
		Board3d->flipBoard();
	else*/
		Board->flipBoard();
}
///////////////////////////////////////
//
//	match::resize
//
///////////////////////////////////////
void match::resize(void)
{
/*	if( Resource->OPTION_3DBoard )
	{
		Board3d->ResizeBoard();
		setFixedSize( Board3d->width(), Board3d->height() );
	}
	else*/
	{
        	if(!paused())
		  Board->ResizeBoard();
		setFixedSize( Board->width(), Board->height() );
	}
}
///////////////////////////////////////
//
//	match::redraw
//
///////////////////////////////////////
void match::redraw(void)
{
/*	if( Resource->OPTION_3DBoard )
		Board3d->redrawAll();
	else*/
        if (!paused())
		Board->redrawAll();
}
///////////////////////////////////////
//
//	match::modified
//
///////////////////////////////////////
bool match::modified(void)
{
	return Modified;
}
///////////////////////////////////////
//
//	match::paused
//
///////////////////////////////////////
bool match::paused(void)
{
	return Paused;
}
///////////////////////////////////////
//
//	match::slot_flagFell
//
///////////////////////////////////////
void match::slot_flagFell( const bool Army )
{
	if( Army == WHITE )
	{
		emit setStatusBar( WHITE_FLAG );
		if( ( Resource->OPTION_Auto_Call_Flag ) && ( WhiteInput != PLAYERLOCAL ) )
			emit sendCMD( CMD_Call_Flag, matchID() );
	}
	else
	{
		emit setStatusBar( BLACK_FLAG );
		if( ( Resource->OPTION_Auto_Call_Flag ) && ( BlackInput != PLAYERLOCAL ) )
			emit sendCMD( CMD_Call_Flag, matchID() );
	}
}
///////////////////////////////////////
//
//	match::flag
//
///////////////////////////////////////
bool match::flag( const bool Army )
{
	return Clock->Flag[Army];
}
///////////////////////////////////////
//
//	match::setPaused
//
///////////////////////////////////////
void match::setPaused( const bool State )
{
	Paused = State;
// 	if( Resource->OPTION_3DBoard )
// 		Board3d->setPaused( State );
// 	else
		Board->setPaused( State );
	if( Paused )
	{
		Clock->Pause();
		emit setStatusBar( PAUSED );
	}
	else
	{
		Clock->Resume();
		if( onMove() == WHITE ) emit setStatusBar( WHITE_TURN );
		else emit setStatusBar( BLACK_TURN );
	}
}
///////////////////////////////////////
//
//	match::setCurrent
//
///////////////////////////////////////
void match::setCurrent( const bool State )
{
	Current = State;
}
///////////////////////////////////////
//
//	match::setDrawOffered
//
///////////////////////////////////////
void match::setDrawOffered( const bool Army )
{
	Draw_Offered[ Army ] = 2;
	if( Army == WHITE )
		emit setStatusBar( WHITE_DRAW_OFFER );
	else
		emit setStatusBar( BLACK_DRAW_OFFER );
}
///////////////////////////////////////
//
//	match::onMove
//
///////////////////////////////////////
char match::onMove(void)
{
	return Logic->OnMove;
}
///////////////////////////////////////
//
//	match::whiteInput
//
//////////////////////////////////////
char match::whiteInput(void)
{
	return WhiteInput;
}
///////////////////////////////////////
//
//	match::blackInput
//
///////////////////////////////////////
char match::blackInput(void)
{
	return BlackInput;
}
///////////////////////////////////////
//
//	match::setModified
//
///////////////////////////////////////
void match::setModified( const bool state )
{
	Modified = state;
}
///////////////////////////////////////
//
//	match::setName
//
///////////////////////////////////////
void match::setName( const bool Army, const QString Name )
{
	if( Army == WHITE ) Record->TAG_White = Name;
	else Record->TAG_Black = Name;
}
///////////////////////////////////////
//
//	match::save
//
///////////////////////////////////////
bool match::save( QString URL )
{
	kdDebug() << "Match::save " << URL << endl;
	Record->whitePlayer = WhiteInput;
	Record->blackPlayer = BlackInput;
	Record->whiteTime = Clock->getCentiseconds(WHITE) / 10;
	Record->blackTime = Clock->getCentiseconds(BLACK) / 10;
	return Record->save( URL );
}
///////////////////////////////////////
//
//	match::load
//
///////////////////////////////////////
bool match::load( const QString URL, const int pos )
{
	Loading = TRUE;
	loadTimer = 20;
	if( !Record->open( URL ) ) return FALSE;
	bool value = Record->load( pos );
	if( !Record->TAG_FEN.isEmpty() )
	{
// We have FEN data, handle it here

//Not for the the time being, later
//	emit sendCMD(CMD_Set_Board,matchID(),Record->TAG_FEN);
//         Logic->setBoardFromFen(Record->TAG_FEN);
         
	}
	return value;
        
        
}
///////////////////////////////////////
//
//	match::loading
//
///////////////////////////////////////

bool match::loading( void )
{
	return Loading;
}
///////////////////////////////////////
//
//	match::loadSpecial
//
///////////////////////////////////////
void match::loadSpecial( void )
{
	/*
			As pointless as this function may seem ( in light of these variables and more
			being setup in tick(), it is required to initialize a Chess Engine and feed it
			the moves from a saved game so you can play it again later.
	*/
	WhiteInput = Record->whitePlayer;
	BlackInput = Record->blackPlayer;
	
	WhiteInput = PLAYERLOAD;
	BlackInput = PLAYERLOAD;
}
///////////////////////////////////////
//
//	match::notation
//
///////////////////////////////////////
QStringList* match::notation( void )
{
	return Record->notation();
}
///////////////////////////////////////
//
//	match::url
//
///////////////////////////////////////
QString match::url( void )
{
	if( !Record->CurrentURL.isEmpty() ) return Record->CurrentURL;
	if( Resource->OPTION_Reuse_PGN ) return Resource->PGN_Filename;
	return QString::null;
}
///////////////////////////////////////
//
//	match::caption
//
///////////////////////////////////////
QString match::caption( void )
{
	return Record->caption();
}
///////////////////////////////////////
//
//	match::clock
//
///////////////////////////////////////
QString match::clock( const bool Army )
{
	if( Army == WHITE ) return Clock->whiteClock;
	return Clock->blackClock;
}
///////////////////////////////////////
//
//	match::centiseconds
//
///////////////////////////////////////
int match::centiseconds( const bool Army )
{
	return Clock->getCentiseconds( Army );
}
///////////////////////////////////////
//
//	match::saveStatusBar
//
///////////////////////////////////////
void match::saveStatusBar( const int ID, const QString& MSG )
{
	StatusCode = ID;
	StatusMessage = MSG;
}
///////////////////////////////////////
//
//	match::resendStatusBar
//
///////////////////////////////////////
void match::resendStatusBar( void )
{
	emit setStatusBar( StatusCode, StatusMessage );
}
///////////////////////////////////////
//
//	match::slot_Select
//
///////////////////////////////////////
void match::slot_Select( int number )
{
	bool preMoving(FALSE);
	char tmp;

	if( Paused ) return;
	/* If a chessman is being previewed, clear that preview. */
	if( preview )
	{
		preview = FALSE;
		for( tmp = 1;tmp < 64;tmp++ )
			if( ( Logic->current[tmp].Note != NOTE_SELECT ) &&
					( Logic->current[tmp].Note != NOTE_NONE ) )
					{
						Logic->current[tmp].Note = NOTE_NONE;
// 						if( Resource->OPTION_3DBoard )
// 							Board3d->drawPosition( tmp );
//						else
							Board->drawPosition( tmp );
					}
		drawPosition( 0 );
	}
	/* Check to make sure it's our turn to select. */
	if( onMove() == BLACK )
	{
		if( ( BlackInput != PLAYERLOCAL ) && ( WhiteInput == PLAYERLOCAL ) && ( Resource->OPTION_Premove ) )
			preMoving = TRUE;
		else
			if( BlackInput != PLAYERLOCAL ) return;
	}
	if( onMove() == WHITE )
	{
		if( ( WhiteInput != PLAYERLOCAL ) && ( BlackInput == PLAYERLOCAL ) && ( Resource->OPTION_Premove ) )
			preMoving = TRUE;
		else
			if( WhiteInput != PLAYERLOCAL ) return;
	}
	/* If you left click on a selected square, it becomes unselected. */
	if( Logic->current[number].Note == NOTE_SELECT )
	{
		Logic->current[number].Note = NOTE_NONE;
		drawPosition( number );
		playSound( SND_SELECT );
		return;
	}
	if( preMoved )
	{
		preMoved = FALSE;
		playSound( SND_SELECT );
// 		if( Resource->OPTION_3DBoard )
// 		{
// 			Board3d->setPremovePositions( Null, Null );
// 			Board3d->drawPosition( Logic->Pointer( preMove.fromFile, preMove.fromRank ) );
// 			Board3d->drawPosition( Logic->Pointer( preMove.toFile, preMove.toRank ) );
// 			Board3d->commit();
// 		}
// 		else
		{
			Board->setPremovePositions( Null, Null );
			Board->drawPosition( Logic->Pointer( preMove.fromFile, preMove.fromRank ) );
			Board->drawPosition( Logic->Pointer( preMove.toFile, preMove.toRank ) );
			Board->commit();
		}
	}
	/* Check to see if there is already a selected square. */
	for( tmp = 0; tmp < 64; tmp++ )
		if( Logic->current[tmp].Note == NOTE_SELECT ) break;
	if( tmp != 64 )
	{
		/*
				If there is already a selected square, and you just clicked on another of your men,
				we'll change your selection.
		*/
		if( Logic->current[number].ManPtr != Null )
			if( Logic->chessman[ Logic->current[tmp].ManPtr ].Army == Logic->chessman[ Logic->current[number].ManPtr ].Army )
			{
				Logic->current[tmp].Note = NOTE_NONE;
				drawPosition( tmp );
				Logic->current[number].Note = NOTE_SELECT;
				drawPosition( number );
				if( Resource->OPTION_Auto_Preview == TRUE )
					slot_Preview( number );
				else
					playSound( SND_SELECT );
				return;
			}
		/* If there is already a selected square, but this one you just clicked
				isn't one of your chessmen, you must want to move a piece. */
		if( ( Record->currentIndex != ( Record->Positions.count() - 1 ) ) && ( Record->currentIndex ) )
		{
			kdDebug() << "NO_MOVE_WHILE_REVIEW" << endl;
			emit setStatusBar( NO_MOVE_WHILE_REVIEW );
			return;
		}
		if( preMoving )
		{
			clearMove( TRUE );
			preMove.fromRank = Logic->current[tmp].Rank;
			preMove.fromFile = Logic->current[tmp].File;
			preMove.toRank = Logic->current[number].Rank;
			preMove.toFile = Logic->current[number].File;
			preMoved = TRUE;
			playSound( SND_SELECT );
// 			if( Resource->OPTION_3DBoard )
// 				Board3d->setPremovePositions( tmp, number );
// 			else
				Board->setPremovePositions( tmp, number );
		}
		else
		{
			clearMove();
			chessMove.fromRank = Logic->current[tmp].Rank;
			chessMove.fromFile = Logic->current[tmp].File;
			chessMove.toRank = Logic->current[number].Rank;
			chessMove.toFile = Logic->current[number].File;
			slot_Move();
		}
		return;
	}
	if( Logic->current[number].ManPtr != Null )
		if( ( Logic->chessman[ Logic->current[number].ManPtr ].Army != onMove() ) && ( preMoving == FALSE ) )
			return;
	tmp = Logic->current[number].ManPtr;
	/* You can't select an empty square. */
	if( tmp == Null ) return;
	/* If your clicking on one of your chessmen, you can select it. */
	clearSelections();
	if( ( Logic->chessman[tmp].Army == WHITE ) && ( WhiteInput == PLAYERLOCAL ) )
		Logic->current[number].Note = NOTE_SELECT;
	if( ( Logic->chessman[tmp].Army == BLACK ) && ( BlackInput == PLAYERLOCAL ) )
		Logic->current[number].Note = NOTE_SELECT;
	drawPosition( number );
	if( Resource->OPTION_Auto_Preview == TRUE )
		slot_Preview( number );
	else
		/* Play the select sound if you selected a square & you didn't preview */
		if( Logic->current[number].Note == NOTE_SELECT )
			playSound( SND_SELECT );
                        
                        
}
///////////////////////////////////////
//
//	match::slot_Preview
//
///////////////////////////////////////
void match::slot_Preview( int number )
{
	char tmp;

	if( Paused ) return;
	clearSelections();
	tmp = Logic->current[number].ManPtr;
	if( tmp == Null ) return;

	preview = TRUE;
	Logic->HashLegal( tmp );

	/* If your clicking on one of your chessmen, you can select it. */
	if( ( Logic->chessman[tmp].Army == WHITE ) && ( WhiteInput == PLAYERLOCAL ) && ( Logic->OnMove == WHITE ) )
		Logic->current[number].Note = NOTE_SELECT;
	if( ( Logic->chessman[tmp].Army == BLACK ) && ( BlackInput == PLAYERLOCAL ) && ( Logic->OnMove == BLACK ) )
		Logic->current[number].Note = NOTE_SELECT;
 	for( tmp = 1;tmp < 64;tmp++ )
 		if( Logic->current[tmp].Note != NOTE_NONE )
// 			if( Resource->OPTION_3DBoard )
// 				Board3d->drawPosition( tmp );
// 			else
				Board->drawPosition( tmp );
	drawPosition( 0 );
	playSound( SND_SELECT );
        
        /*if( Logic->Army1CheckedKingPos != Null )
	{
		Logic->current[ Logic->Army1CheckedKingPos ].Note = NOTE_ATTACK;
		Board->drawPosition( Logic->Army1CheckedKingPos );
		Board->commit();
	}
             
        if( Logic->Army2CheckedKingPos != Null )
	{
		Logic->current[ Logic->Army2CheckedKingPos ].Note = NOTE_ATTACK;
		Board->drawPosition( Logic->Army2CheckedKingPos );
		Board->commit();
	}*/ 
}
///////////////////////////////////////
//
//	match::setBoard
//
///////////////////////////////////////
void match::setBoard( const ICSPosition pos )
{
	QString newSAN( pos.SAN );
	Movelist = FALSE;
	/* Do NOT accept a setBoard on a concluded match */
	if( Logic->OnMove == -1 ) return;
	Logic->OnMove = !pos.OnMove;
	/* Set the Match Mode */
	ICSGameMode = pos.Mode;
	/* Set the board */
	Logic->setBoard( pos.Board, pos.PawnPushFile );
	Logic->MoveCounter = pos.MoveCounter;
	/* Remove symbols from the server's SAN that will be re-appended by Knights */
	newSAN.replace( QRegExp("\\+"), "" );
	newSAN.replace( QRegExp("#"), "" );
	/* create a ChessMove from the notation */
	strcpy( Logic->chessMove.CAN, pos.CAN.latin1() );
	strcpy( Logic->chessMove.SAN, newSAN.latin1() );
	Logic->parseCAN( Logic->OnMove );
	chessMove = Logic->chessMove;
	/* Update record information */
	Record->TAG_White = pos.WhiteName;
	Record->TAG_Black = pos.BlackName;
	/* Run through the Move function */
	if( !JustMoved )
		slot_Move( TRUE );
	else
		JustMoved = FALSE;
	Clock->Set( pos.WhiteTime, pos.BlackTime, pos.OnMove );
	if( pos.ClockTicking ) Clock->Resume();
	else Clock->Pause();
	Logic->OnMove = pos.OnMove;
}
///////////////////////////////////////
//
//	match::slot_Move
//
///////////////////////////////////////
bool match::slot_Move( ChessMove newMove )
{
	Logic->chessMove = newMove;
	if( Logic->parseCAN( onMove() ) != TRUE )
		if( Logic->parseSAN() != TRUE )
 		{
 			//kdDebug() << "#" << matchID() << ": Recieved bad data: " << newMove.CAN << endl;
			return FALSE;
 		}
	chessMove = Logic->chessMove;
	return slot_Move();
}
bool match::slot_Move( const bool FromServer )
{
	int result(Null);
	char fromPtr(Null), toPtr(Null), manPtr(Null);
	QString SAN;

	//kdDebug() << "match::slot_Move: Acting upon CAN: " << chessMove.CAN << " SAN: " << chessMove.SAN << endl;
	/* Bring the display up to the latest move before we proceed */
	if( Record->Positions.count() && !FromServer )
		review( Record->Moves.count() - 1 );

	fromPtr = ( chessMove.fromRank * 8 ) + chessMove.fromFile;
	toPtr = ( chessMove.toRank * 8 ) + chessMove.toFile;
	/* Skip all this if we're being called due to a server move */
	if( !FromServer )
	{
		if( ( fromPtr > 63 ) || ( toPtr > 63 ) ) return FALSE;
		if( ( fromPtr < 0 ) || ( toPtr < 0 ) ) return FALSE;
		manPtr = Logic->current[fromPtr].ManPtr;
		if( manPtr == Null ) return FALSE;
		/* Make the move */
		Logic->chessMove = chessMove;
		if( Logic->Move() == FALSE ) return FALSE;
		chessMove = Logic->chessMove;
	}
	/* Check to see if the game is ended */
	/* 50 Move Rule? */
	if( Logic->MoveCounter == 50 )
		result = CMD_Result_Draw;
	/*
			A Draw via 3 move repeat
	*/
/*	if( Record->isThreeMoveDraw() )
	{
		result = CMD_Result_Draw;
		Record->TAG_Result = "1/2-1/2";
		emit setStatusBar( GAME_DRAW );
	}      */
	/*
			A Draw?

			Condition 1 = White Material Draw
			Condition 2 = Black Material Draw
			Condition 3 = Both sides agree to a draw
	*/
// 	if( Logic->isDraw( WHITE ) || Logic->isDraw( BLACK ) ||
// 		( Draw_Offered[WHITE] && Draw_Offered[BLACK] ) )
             
             
             if ((Logic->OnMove==BLACK &&  Logic->isDraw(WHITE)) || (Logic->OnMove==WHITE &&  Logic->isDraw(BLACK)) || ( Draw_Offered[WHITE] && Draw_Offered[BLACK] ))
             {
                 kdDebug() << "isDraw returned true" << endl;
                 result = CMD_Result_Draw;
             }
	/* Is White under check? */
	if( Logic->isCheck( WHITE ) )
	{
		if( !Logic->isLegal( WHITE ) )
		{
			SAN += '#';
			result = CMD_Result_Black;
		}
		else
		{
			SAN += '+';
			playSound( SND_CHECK );
                        kdDebug() << "Settings status bar: WHITE_CHECKED" << endl;
                        emit setStatusBar(WHITE_CHECKED);
		}
	}
	/* Is Black under check? */
	if( Logic->isCheck( BLACK ) )
	{
  	if( !Logic->isLegal( BLACK ) )
  	{
  		SAN += '#';
			result = CMD_Result_White;
  	}
		else
		{
			SAN += '+';
			playSound( SND_CHECK );
                        kdDebug() << "Settings status bar: BLACK_CHECKED" << endl;
                        emit setStatusBar(BLACK_CHECKED);
		}
	}
	/* Clear selections from the board */
 	clearSelections();
	/* Check to make sure this isn't the starting position in an ICS match */
	if( QString( chessMove.SAN ) != "none" )
	{
		if( !SAN.isEmpty() ) strcat( chessMove.SAN, SAN.latin1() );
		if( ( !FromServer ) && ( !Movelist ) ) emit moved( chessMove, matchID() );
		Record->Positions << Logic->board();
		Record->Moves << chessMove;
		Record->currentIndex = Record->Moves.count() - 1;
		/* Draw Last Move Hints */
	 	if( Resource->OPTION_Show_Last_Move )
	 	{
		 	Logic->current[fromPtr].Note = NOTE_MOVE;
			if( QString( chessMove.SAN ).contains( QChar( 'x' ), FALSE ) )
			 	Logic->current[toPtr].Note = NOTE_ATTACK;
			else
			 	Logic->current[toPtr].Note = NOTE_MOVE;
// 			if( Resource->OPTION_3DBoard )
// 			{
// 			 	Board3d->drawPosition( fromPtr );
// 			 	Board3d->drawPosition( toPtr );
// 				Board3d->commit();
// 			}
//			else
			{
			 	Board->drawPosition( fromPtr );
			 	Board->drawPosition( toPtr );
				Board->commit();
			}
		}
	}
	emit setNotation();
	/* Take care of changing turns, status messages, etc. */
	if( result == Null )
	{
		Clock->Moved();
  	Logic->OnMove = !Logic->OnMove;
  	if( Logic->OnMove == WHITE ) emit setStatusBar( WHITE_TURN );
  	if( Logic->OnMove == BLACK ) emit setStatusBar( BLACK_TURN );
		if( Draw_Offered[WHITE] ) Draw_Offered[WHITE]--;
		if( Draw_Offered[BLACK] ) Draw_Offered[BLACK]--;
		if( chessMove.NAG != 0 )
				emit setStatusBar( COMMENT + chessMove.NAG );
		if( SAN.right(1) != "+" ) playSound( SND_MOVE );
	}
	else
	{
		endMatch( result );
		emit sendCMD( result, matchID() );
		if( Logic->MoveCounter == 50 )
			emit setStatusBar( GAME_50_MOVES );
		playSound( SND_MATCH_OVER );
	}
	/* Handle special cases */
	if( loading() ) loadTimer = 10; // Loading...
	else Modified = TRUE;
	if( !FromServer ) // From ICS...
		JustMoved = TRUE;
	if( preMoved )
	{
		preMoved = FALSE;
// 		if( Resource->OPTION_3DBoard )
// 			Board3d->setPremovePositions( Null, Null );
//		else
			Board->setPremovePositions( Null, Null );
		chessMove = preMove;
		slot_Move();
	}
	if( ( ( Logic->OnMove == WHITE ) && ( whiteInput() == PLAYEREMAIL ) ) ||
			( ( Logic->OnMove == BLACK ) && ( blackInput() == PLAYEREMAIL ) ) )
			{
				Clock->Pause(); // Email
// 				KMessageBox::questionYesNo( this,
// 																		i18n("Would you like to email this move?"),
// 																		i18n("Send Email?") );
			}
                        
	if( Logic->Army1CheckedKingPos != Null )
	{
		Logic->current[ Logic->Army1CheckedKingPos ].Note = NOTE_ATTACK;
		Board->drawPosition( Logic->Army1CheckedKingPos );
		Board->commit();
	}
             
        if( Logic->Army2CheckedKingPos != Null )
	{
		Logic->current[ Logic->Army2CheckedKingPos ].Note = NOTE_ATTACK;
		Board->drawPosition( Logic->Army2CheckedKingPos );
		Board->commit();
	}           
	return TRUE;
}
///////////////////////////////////////
//
//	match::retract
//
///////////////////////////////////////
void match::retract( void )
{
  		emit setStatusBar(ILLEGAL_MOVE);
			review( Record->Moves.count() - 2);
			_retract();
}
void match::_retract( void )
{
	if( Record->Moves.count() )
	{
        	int x = Logic->OnMove;
        		Record->Moves.remove( Record->Moves.at( Record->Moves.count() -1 )  );
		Record->Positions.remove( Record->Positions.at( Record->Positions.count() -1 ) );
		Record->currentIndex = Record->Moves.count() -1;
                if (x==WHITE)
                 Logic->OnMove = BLACK;
                if (x==BLACK)
                 Logic->OnMove = WHITE;
                if( Logic->OnMove == WHITE ) emit setStatusBar( WHITE_TURN );
        if( Logic->OnMove == BLACK ) emit setStatusBar( BLACK_TURN );
	}
}
///////////////////////////////////////
//
//	match::review
//
///////////////////////////////////////
void match::review( const int Index )
{
	int tmp, step;
	QString tmpS;
	if( ICSGameMode == ICS_Examine )
	{
		/* Examined Game: Use Examine Symantics */
		if( Index < (signed)Record->currentIndex ) step = -1;
		else step = 1;
		if( ( Record->currentIndex == 0 ) && ( Index == 0 ) && ( step == 1 ) )
			emit ( CMD_Examine_Forward, matchID() );
		for( tmp = Record->currentIndex; tmp != Index; tmp += step )
			if( step == 1 ) emit sendCMD( CMD_Examine_Forward, matchID() );
			else
			{
				_retract();
				_retract();
				emit sendCMD( CMD_Examine_Backward, matchID() );
			}
	}
	else
	{
		/* Regular Game: Use Standard Review Symantics */
		if( Record->Positions.count() == 0 ) return;
		if( Index < 0 ) return;
		if( Index > ( (signed int)Record->Positions.count() - 1 ) ) return;
		Logic->setBoard( Record->Positions[Index] );
		clearSelections();
		/* Statusbar shows NAG if available, otherwise Army to move */
		if( Record->Moves[Index].NAG )
		{
			tmp = StatusCode;
			tmpS = StatusMessage;
			emit setStatusBar( COMMENT + Record->Moves[Index].NAG );
			StatusCode = tmp;
			StatusMessage = tmpS;
		}
		else
			emit setStatusBar( StatusCode, StatusMessage );
		Record->currentIndex = Index;
	}
}
///////////////////////////////////////
//
//	match::endMatch
//
///////////////////////////////////////
void match::endMatch( const int result, const bool Army )
{
		Clock->Pause();
 		Logic->OnMove = -1;	
		switch( result )
		{
			case CMD_Result_White:
	  		emit setStatusBar( WHITE_CHECKMATE );
				Record->TAG_Result = "1-0";
				Record->TAG_Termination = "Normal";
				break;
			case CMD_Result_Black:
				emit setStatusBar( BLACK_CHECKMATE );
				Record->TAG_Result = "0-1";
				Record->TAG_Termination = "Normal";
				break;
			case CMD_Result_Draw:
				emit setStatusBar( GAME_DRAW );
				Record->TAG_Result = "1/2-1/2";
				break;
			case CMD_Resign:
                        	kdDebug() << "CMD_Resign" << endl;
				// For Resignations, the listed army must be the victor.
				if( Army == WHITE )
				{
					emit setStatusBar( BLACK_RESIGN );
					Record->TAG_Result = "1-0";
					Record->TAG_Termination = "Black resigned";
				}
				else
				{
					emit setStatusBar( WHITE_RESIGN );
					Record->TAG_Result = "0-1";
					Record->TAG_Termination = "White resigned";
				}
				break;
			case CMD_Call_Flag:
				if( Army == WHITE )
				{
					emit setStatusBar( WHITE_CALL_FLAG );
					Record->TAG_Result = "1-0";
					Record->TAG_Termination = "Black's flag was called";
				}
				else
				{
					emit setStatusBar( BLACK_CALL_FLAG );
					Record->TAG_Result = "0-1";
					Record->TAG_Termination = "White's flag was called";
				}
				break;
			case CMD_Lost_Contact:
				emit setStatusBar( LOST_CONTACT );
				Record->TAG_Result = "*";
				Record->TAG_Termination = "Lost Contact with Opponent";
				break;
			default:
				break;
		}
}
///////////////////////////////////////
//
//	match::playSound
//
///////////////////////////////////////
void match::playSound( const int snd )
{
	if( Resource->OPTION_Audio_Current_Only && !Current ) return;
	Resource->play( snd );
}
///////////////////////////////////////
//
//	match::drawPosition
//
///////////////////////////////////////
void match::drawPosition( const int number )
{
// 		if( Resource->OPTION_3DBoard )
// 		{
// 			Board3d->drawPosition( number );
// 			 ->commit();
// 		}
// 		else
		{
			Board->drawPosition( number );
			Board->commit();
		}
}
///////////////////////////////////////
//
//	match::readMovelist
//
///////////////////////////////////////
void match::readMovelist( const QString Inbound )
{
	if( Inbound.isEmpty() ) return;
	kdDebug() << "Processing: " << Inbound << endl;
	Movelist = TRUE;
	QStringList fields = QStringList::split( QChar(' '), Inbound, FALSE );
	/* create White ChessMove from the notation */
	if( fields.count() < 2 ) return;
	strcpy( Logic->chessMove.SAN, fields[1].latin1() );
	Logic->parseSAN();
	chessMove = Logic->chessMove;
	if( !slot_Move( FALSE ) ) kdDebug() << fields[1] << " failed from movelist." << endl;
	/* create Black ChessMove from the notation */
	if( fields.count() < 4 ) return;
	strcpy( Logic->chessMove.SAN, fields[3].latin1() );
	Logic->parseSAN();
	chessMove = Logic->chessMove;
	if( !slot_Move( FALSE ) ) kdDebug() << fields[3] << " failed from movelist." << endl;
}
