/***************************************************************************
                          match.h  -  description
                             -------------------
    begin                : Mon Jul 2 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MATCH_H
#define MATCH_H

#include <qwidget.h>
#include "resource.h"
#include "chessclock.h"
//#include "board3d.h"
#include "board.h"
#include "logic.h"
#include "pgn.h"

class match : public QWidget
{
   Q_OBJECT
	public:
									match(QWidget *parent=0, const char *name=0, resource *Rsrc=0);
									~match();
		int						matchID( void );
		bool					modified( void );
		bool					paused( void );
		bool					loading( void );
		bool					flag( const bool );
		char					onMove( void );
		char					whiteInput( void );
		char					blackInput( void );
		QString				caption( void );
		QString				url( void );
		void					flip( void );
		void					tick( void );
		void					retract( void );
		void					resendStatusBar( void );
		void					setMatchID( const int ID );
		void					setVisibility( const bool vis );
		void					setPaused( const bool State );
		void					setCurrent( const bool State );
		void					setDrawOffered( const bool Army );
		void					setInput( const char White, const char Black );
		void					setName( const bool Army, const QString Name );
		void					setBoard( const ICSPosition pos );
		void					endMatch( const int, const bool Amry=WHITE );
		void					readMovelist( const QString );
                
	public slots:
		void					resize( void );
		void					redraw( void );
		QString				clock( bool );
		int						centiseconds( bool );
		bool					save( QString );
		bool					load( const QString URL, const int pos=0 );
		void					loadSpecial( void );
		QStringList*	notation( void );
		void					setModified( const bool );
		void 					slot_Select( int );
		void 					slot_Preview( int );
		bool					slot_Move( ChessMove );
		bool					slot_Move( const bool FromServer=FALSE );
		void					slot_flagFell( const bool );
		void					review( const int );
		void					saveStatusBar( const int, const QString& );
	signals:
		void					setNotation( void );
		void					setClocks( void );
		void					setStatusBar( const int ID, const QString& MSG=QString::null );
		void					moved( const ChessMove, const int );
		void					sendCMD( const int, const int, const QString& value = QString::null );
	protected:
		void 					clearSelections( const char fromPtr=Null );
		void 					clearMove( const bool pre=FALSE );
		void					drawPosition( const int );
		void					playSound( const int );
		void					_retract( void );
	private:
		resource			*Resource;
		board					*Board;
		//dboard3d				*Board3d;
		pgn						*Record;
		logic					*Logic;
		chessclock		*Clock;
		ChessMove			chessMove;
		ChessMove			preMove;
		bool					preMoved;
		bool					preview;
		bool					Modified;
		bool					Paused;
		bool					Loading;
		bool					JustMoved;
		bool					Current;
		bool					Movelist;
		int						MatchID;
		int						StatusCode;
		int						loadTimer;
		QString				StatusMessage;
		char					ICSGameMode;
		char					WhiteInput;
		char					BlackInput;
		char					Draw_Offered[2];
};

#endif
