/***************************************************************************
                          pgn.h  -  description
                             -------------------
    begin                : Mon Jul 30 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PGN_H
#define PGN_H

#include <klocale.h>
//#include <ktempfile.h>
//#include <kio/netaccess.h>
#include <qobject.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qvaluelist.h>
#include <qfile.h>
#include <qtextstream.h>
#include "definitions.h"
#include "resource.h"

/**
  *@author Troy Corbin Jr.
  */

typedef struct gameItem
{
	int pos;
	QString Title;
	QString Date;
	QString Site;
	QString Round;
	QString Result;
};

typedef QValueList<ChessMove> MoveList;

class pgn : public QObject
{
   Q_OBJECT
	public:
	/* The Seven */
	QString 						TAG_Event;
	QString 						TAG_Site;
	QString 						TAG_Date;
	QString 						TAG_Round;
	QString 						TAG_White;
	QString 						TAG_Black;
	QString 						TAG_Result;
	/* Event Related */
	QString 						TAG_EventDate;
	QString 						TAG_EventSponsor;
	QString 						TAG_Section;
	QString 						TAG_Stage;
	QString 						TAG_Board;
	/* Player Related */
	QString 						TAG_WhiteTitle;
	QString 						TAG_WhiteElo;
	QString 						TAG_WhiteUSCF;
	QString 						TAG_WhiteNA;
	QString 						TAG_WhiteType;
	QString 						TAG_BlackTitle;
	QString 						TAG_BlackElo;
	QString 						TAG_BlackUSCF;
	QString 						TAG_BlackNA;
	QString 						TAG_BlackType;
	/* Opening Info */
	QString 						TAG_Opening;
	QString 						TAG_Variation;
	QString 						TAG_SubVariation;
	QString 						TAG_ECO;
	QString 						TAG_NIC;
	/* Time and Date */
	QString 						TAG_Time;
	QString 						TAG_UTCTime;
	QString 						TAG_UTCDate;
	/* Misc */
	QString 						TAG_TimeControl;
	QString 						TAG_SetUp;
	QString 						TAG_FEN;
	QString					 		TAG_Termination;
	QString					 		TAG_Annotator;
	QString					 		TAG_Mode;
	QString 						TAG_PlyCount;

											/* These are used in unfinished save games */
	int									whitePlayer;
	int									blackPlayer;
	int									whiteTime;
	int									blackTime;
	TCPList							whiteTCP;
	TCPList							blackTCP;

	QString							CurrentURL;
	gameItem						scannedMatch;
	MoveList						Moves;
	MoveList						RAV;
	QStringList					Positions;
	QStringList					Annotation;
	unsigned int				currentIndex;

											pgn( resource *Rsrc=0 );
											~pgn();
	void								clear( void );
	void								init( void );
	int									scan( void );
	bool								load( const int pos=0 );
	bool								save( QString URL );
	bool								open( const QString& URL );
	QStringList*				notation( void );
	void								close( void );

	QString							getNAG( int num );
	QString							caption( void );

	signals:

	void								processMove( ChessMove );
	void								processSpecial( void );

	public slots:

	bool								loadNext( void );

	private:

	bool								FLAG_ReadingRAV;
	resource						*Resource;
	QString							tempFile;
	QString							currentLine;
	QFile								File;
	QTextStream					Input;

	QChar								getch( void );
	QString							getword( void );
	void								clearTags( void );
	void								Parse_Tag( void );
	void								Parse_Annotation( void );
	void								Parse_Knights_Data( void );
};

#endif
