/***************************************************************************
                          resource.h  -  description
                             -------------------
    begin                : Tue Jul 17 2001
    copyright            : (C) 2001, 2002 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RESOURCE_H
#define RESOURCE_H


/**
  *@author Troy Corbin Jr.
  */

/* KDE */
//#include <kapplication.h>
#include <kiconloader.h>
#include <kconfig.h>
#include <klocale.h>
//#include <kstandarddirs.h>
#include <klistview.h>
//#include <kcmdlineargs.h>
/* Qt */
#include <qstylesheet.h>
#include <qvaluelist.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qdir.h>
#include <qimage.h>
#include <qpixmap.h>
/* Local */
#include "definitions.h"
#include "knightspixcache.h"
//#include "audio.h"

typedef struct engineResource
{
		QListViewItem	*Item;
		QString				Name;
		QString				Filename;
		QString				Arguments;
		QString				LogFile;
		int						Protocol;
		int						Wins;
		int						Losses;
		int						Draws;
		int						CurrentRef;
                bool Book;
                bool Hint;
                bool Undo;
                QString very_easy_args;
                QString easy_args;
                QString moderate_args;
                QString hard_args;
                QString very_hard_args;
};

typedef struct serverResource
{
		QListViewItem *Item;
		QString				Name;
		QString				URL;
		QString				UserName;
		QString				Password;
		QString				LogFile;
		QString				Timeseal;
		bool					StorePass;
		int						Port;
		int						CurrentRef;
};

typedef QValueList<engineResource> engineList;
typedef QValueList<serverResource> serverList;

class resource
{
	public:
													/* The current Theme */
		int										ThemeSize;
                bool									ThemeBorder;
		QStyleSheet						*ConsoleStyle;
		QString								CurrentBoard;
		QString								CurrentChessmen;
		QString								CurrentAudio;
		ThemeHeader						boardHeader;
		ThemeHeader						chessmenHeader;
		ThemeHeader						audioHeader;
		QPixmap								SquareLight;
		QPixmap								SquareDark;
		QPixmap								HighlightSelect;
		QPixmap								HighlightMove;
		QPixmap								HighlightAttack;
		QPixmap								BlackKing;
		QPixmap								BlackQueen;
		QPixmap								BlackBishop;
		QPixmap								BlackKnight;
                
                QPixmap                                                         Border;
                QPixmap                                                         BorderLightOn;
                QPixmap                                                         BorderLightOff;
		QPixmap								BlackRook;
		QPixmap								BlackPawn;
		QPixmap								WhiteKing;
		QPixmap								WhiteQueen;
		QPixmap								WhiteBishop;
		QPixmap								WhiteKnight;
		QPixmap								WhiteRook;
		QPixmap								WhitePawn;
		KnightsPixCache				*pixCache;
													/* General Settings */
		QString								Local_Player;
		int										Config_Version;
		int										OPTION_On_Init;
		int										Audio_Volume;
		bool 									OPTION_Audio;
		bool									OPTION_Audio_Current_Only;
		bool 									OPTION_Auto_Queen;
		bool 									OPTION_Auto_Preview;
		bool 									OPTION_Auto_Call_Flag;
		bool									OPTION_Auto_Close_Last_ICS;
		bool									OPTION_Book_White;
		bool									OPTION_Book_Black;
		bool									OPTION_Board_Orientation;
		bool									OPTION_High_Quality;
		bool 									OPTION_Play_Black;
		bool									OPTION_Ponder;
		bool									OPTION_Show_Coord;
		bool									OPTION_Show_Last_Move;
		bool									OPTION_Show_Splash;
		bool									OPTION_Show_Extended_PGN;
		bool									OPTION_Pause_On_Minimize;
		bool									OPTION_Reuse_PGN;
		bool									OPTION_3DBoard;
		QString								PGN_Filename;
		QString								Email_Command_Line;
		int										Widget_Height;
													/* ICS Options */
		int										Seek_Timer;
		int										OPTION_Profanity;
		bool									OPTION_Premove;
		bool									OPTION_Kibitz;
		bool									OPTION_Private;
		bool									OPTION_Shout;
		bool									OPTION_Tell;
		bool									OPTION_Seek;
													/* Console Attributes */
		bool									OPTION_Console_Attach;
		int										OPTION_Console_Orientation;
		int										Console_Splitter_1;
		int										Console_Splitter_2;
		QPoint								Console_Detached_Loc;
		QSize									Console_Detached;
		QSize									Console_Right;
		QSize									Console_Bottom;
													/* Notification Prompts */
		QString								PromptForSaving;
													/* Colors */
		QColor								COLOR_White;
		QColor								COLOR_Black;
		QColor								COLOR_Background;
		QColor								COLOR_Standard;
		QColor								COLOR_PrivateTell;
		QColor								COLOR_ChannelTell;
		QColor								COLOR_Shout;
		QColor								COLOR_Whisper;
		QColor								COLOR_Notification;
		QColor								COLOR_Notation;
		QColor								COLOR_Notation_Shadow;
													/* Fonts */
		QFont									FONT_Standard;
		QFont									FONT_PrivateTell;
		QFont									FONT_ChannelTell;
		QFont									FONT_Shout;
		QFont									FONT_Whisper;
		QFont									FONT_Notification;
													/* Chess Engines */
		//engineResource				engine;
                QString currentEngineName;
                engineResource currentEngine;
                engineList						engines;
		engineList::Iterator	enginesIT;
													/* Chess Servers */
		serverList						servers;
		serverList::Iterator	serversIT;
													/* Time Control Periods */
		TCPList								TCPWhite;
		TCPList								TCPBlack;

//													resource( KCmdLineArgs *args );
													resource();
													~resource();

 		QString difficulty;
                                                                                                        
		void									ConfigRead( void );
		void									ConfigWrite( void );

		void									ReadEngines( void );
		void									WriteEngines( void );

		void									ReadServers( void );
		void									WriteServers( void );
		
		void									ReadTCP( void );
		void									WriteTCP( void );
		
		void									ReadColors( void );
		void									WriteColors( void );

		void									ReadFonts( void );
		void									WriteFonts( void );
		
		void									readThemeDir( void );
		QString								getBoard( int Index=-1 );
		QString								getChessmen( int Index=-1 );
		QString								getSounds( int Index=-1 );
		QString								themeDir( void );
		void									setTheme( int BoardIndex=-1, int ChessmenIndex=-1 );
		void									setAudio( int AudioIndex=-1 );

													/**
													Call buildStyle whenever you need to reparse the Resource configuration */
		void									buildStyle( void );
		QPixmap								LoadIcon( QString Name, int Group );
		void									play( const char );
                void setCurrentBoard(const QString &board) {
                CurrentBoard=board;
                }
                
                void setCurrentChessmen(const QString &men)
                {
                CurrentChessmen=men;
                }
                
                void setCurrentEngineName(const QString &engine)
                {
                currentEngineName=engine;
                }
              

	private:

		KConfig *CFG;
                QStyleSheetItem				*SSI_Default;
		QStyleSheetItem				*SSI_PrivateTell;
		QStyleSheetItem				*SSI_ChannelTell;
		QStyleSheetItem				*SSI_Shout;
		QStyleSheetItem				*SSI_Whisper;
		QStyleSheetItem				*SSI_Notification;

		QString								GlobalDataDir;
		QString								LocalDataDir;

		QStringList						Boards;
		QStringList						Chessmen;
		QStringList						Sounds;
		KIconLoader						*Icons;
		//audio									*Audio;

		/**
			Borrowed from KMail
			Very primitive en/de-cryption so that the password is not
			readable in the config file. But still very easy breakable.
		*/
		QString								encryptStr( const QString& inStr ) const;
		QString								decryptStr( const QString& inStr ) const;
		QString								themeURL( const QString );
		void									loadThemeItem( const QString& URL, QPixmap& Pixmap );
};

#endif
